#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.reference.pictogram.interfaces import IPictogramManager, \
    IPictogramManagerTarget, IPictogramTable, PICTOGRAM_MANAGER_KEY, SELECTED_PICTOGRAM_VOCABULARY
from pyams_i18n.interfaces import II18n
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import query_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config


@factory_config(IPictogramManager)
class PictogramManager(Persistent, Contained):
    """Pictogram manager settings persistent class"""

    selected_pictograms = FieldProperty(IPictogramManager['selected_pictograms'])


@adapter_config(required=IPictogramManagerTarget,
                provides=IPictogramManager)
def pictogram_manager_factory(target):
    """Pictogram manager factory"""
    return get_annotation_adapter(target, PICTOGRAM_MANAGER_KEY, IPictogramManager)


@vocabulary_config(name=SELECTED_PICTOGRAM_VOCABULARY)
class SelectedPictogramsVocabulary(SimpleVocabulary):
    """Selected pictograms vocabulary"""

    def __init__(self, context=None):
        terms = []
        table = query_utility(IPictogramTable)
        if table is not None:
            request = check_request()
            target = get_parent(context, IPictogramManagerTarget)
            if target is not None:
                manager = IPictogramManager(target)
                pictograms = [table.get(name) for name in manager.selected_pictograms or ()]
                terms = [
                    SimpleTerm(v.__name__,
                               title=II18n(v).query_attribute('title', request=request))
                    for v in pictograms if v is not None
                ]
            else:
                terms = [
                    SimpleTerm(v.__name__,
                               title=II18n(v).query_attribute('title', request=request))
                    for v in table.values()
                ]
            terms = sorted(terms, key=lambda x: x.title)
        super(SelectedPictogramsVocabulary, self).__init__(terms)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.zmi.container module

This module provides a few components used for management of associations containers.
"""

from pyramid.view import view_config
from zope.interface import implementer

from pyams_content.component.association import IAssociationContainer, IAssociationContainerTarget
from pyams_content.component.association.interfaces import IAssociationInfo, IAssociationParagraph
from pyams_content.component.association.zmi import IAssociationsTable
from pyams_content.component.association.zmi.interfaces import IAssociationsContainerEditForm
from pyams_content.component.paragraph.zmi.helper import get_json_paragraph_toolbar_refresh_event
from pyams_content.shared.common.interfaces.types import ITypedSharedTool
from pyams_content.shared.common.zmi.types import ISharedToolTypesTable
from pyams_form.ajax import ajax_form_config
from pyams_form.interfaces.form import IInnerSubForm
from pyams_layer.interfaces import IPyAMSLayer
from pyams_layer.skin import apply_skin
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.view import IModalDisplayForm
from pyams_skin.interfaces.viewlet import IContentSuffixViewletManager
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalDisplayForm
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle, IPropertiesEditForm
from pyams_zmi.skin import AdminSkin
from pyams_zmi.table import ActionColumn, ContentTypeColumn, I18nColumnMixin, InnerTableAdminView, \
    NameColumn, ReorderColumn, SortableTable, TableGroupSwitcher, TrashColumn, VisibilityColumn
from pyams_zmi.utils import get_object_hint, get_object_label


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(name='associations',
                required=(ITypedSharedTool, IAdminLayer, ISharedToolTypesTable),
                provides=IColumn)
class SharedToolTypesAssociationsColumn(ActionColumn):
    """Shared tool data types table associations column"""

    hint = _("Default links and external files")
    icon_class = 'fas fa-link'

    href = 'associations-modal.html'
    modal_target = True

    weight = 450


#
# Associations table modal viewer
#

@ajax_form_config(name='associations-modal.html',
                  context=IAssociationContainerTarget, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
@implementer(IAssociationsContainerEditForm)
class AssociationsModalEditForm(AdminModalDisplayForm):
    """Associations modal edit form"""

    modal_class = 'modal-xl'

    subtitle = _("Links and external files")


@adapter_config(required=(IAssociationContainerTarget, IAdminLayer, IModalDisplayForm),
                provides=IFormTitle)
def association_container_display_form_title(context, request, form):
    """Association container display form title"""
    hint = get_object_hint(context, request, form)
    label = get_object_label(context, request, form)
    return TITLE_SPAN_BREAK.format(hint, label)


@factory_config(IAssociationsTable)
class AssociationsTable(SortableTable):
    """Associations table"""

    container_class = IAssociationContainer

    display_if_empty = True


@adapter_config(required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IValues)
class AssociationsTableValues(ContextRequestViewAdapter):
    """Associations table values"""

    @property
    def values(self):
        """Associations container values getter"""
        yield from IAssociationContainer(self.context).values()


@adapter_config(name='reorder',
                required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class AssociationsReorderColumn(ReorderColumn):
    """Associations reorder column"""


@view_config(name='reorder.json',
             context=IAssociationContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def reorder_associations_table(request):
    """Reorder associations"""
    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success',
        'closeForm': False
    }


@adapter_config(name='visible',
                required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class AssociationsVisibleColumn(VisibilityColumn):
    """Associations table visible column"""


@view_config(name='switch-visible-item.json',
             context=IAssociationContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_visible_item(request):
    """Switch visible item"""
    return switch_element_attribute(request)


@adapter_config(name='icon',
                required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class AssociationsIconColumn(ContentTypeColumn):
    """Associations table icon column"""


@adapter_config(name='label',
                required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class AssociationsLabelColumn(NameColumn):
    """Associations table label column"""

    i18n_header = _("Public label")


@adapter_config(name='target',
                required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class AssociationsTargetColumn(I18nColumnMixin, GetAttrColumn):
    """Associations table target column"""

    i18n_header = _("Internal target")
    weight = 50

    def get_value(self, obj):
        """Column value getter"""
        info = IAssociationInfo(obj, None)
        if info is None:
            return MISSING_INFO
        return info.inner_title


@adapter_config(name='size',
                required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class AssociationsSizeColumn(I18nColumnMixin, GetAttrColumn):
    """Associations table size column"""

    i18n_header = _("Size")
    weight = 60

    def get_value(self, obj):
        """Column value getter"""
        info = IAssociationInfo(obj, None)
        if info is None:
            return MISSING_INFO
        return info.human_size


@adapter_config(name='trash',
                required=(IAssociationContainer, IAdminLayer, IAssociationsTable),
                provides=IColumn)
class AssociationsTrashColumn(TrashColumn):
    """Associations table trash column"""


@view_config(name='delete-element.json',
             context=IAssociationContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def delete_data_type(request):
    """Delete data type"""
    result = delete_container_element(request)
    apply_skin(request, AdminSkin)
    event = get_json_paragraph_toolbar_refresh_event(request.context, request)
    if event is not None:
        result.setdefault('callbacks', []).append(event)
        result.setdefault('handle_json', True)
    return result


#
# Main associations table
#

@viewlet_config(name='associations-table',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IAssociationsContainerEditForm,
                manager=IContentSuffixViewletManager, weight=10)
class AssociationsTableView(InnerTableAdminView):
    """Associations table view"""

    table_class = IAssociationsTable
    table_label = _("Links and external files list")

    container_intf = IAssociationContainer


#
# Associations table group
#

@adapter_config(name='associations-group',
                required=(IAssociationContainerTarget, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm, force_implements=False)
class AssociationsGroup(TableGroupSwitcher):
    """Associations table group"""

    legend = _("Links and external files")

    table_class = IAssociationsTable
    container_intf = IAssociationContainer

    weight = 20

    @property
    def state(self):
        """Always open switcher in associations paragraphs"""
        if IAssociationParagraph.providedBy(self.context):
            return 'open'
        return super().state



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.feature.search.interfaces import ISearchManagerInfo
from pyams_sequence.reference import get_reference_target
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.zodb import volatile_property

SEARCH_MANAGER_INFO_KEY = 'pyams_content.search'


@factory_config(ISearchManagerInfo)
class SearchManagerInfo(Persistent, Contained):
    """Search manager persistent info"""

    _reference = FieldProperty(ISearchManagerInfo['reference'])
    name = FieldProperty(ISearchManagerInfo['name'])
    description = FieldProperty(ISearchManagerInfo['description'])

    enable_tags_search = FieldProperty(ISearchManagerInfo['enable_tags_search'])
    _tags_search_target = FieldProperty(ISearchManagerInfo['tags_search_target'])

    enable_collections_search = FieldProperty(ISearchManagerInfo['enable_collections_search'])
    _collections_search_target = FieldProperty(ISearchManagerInfo['collections_search_target'])

    # main search target

    @property
    def reference(self):
        return self._reference

    @reference.setter
    def reference(self, value):
        self._reference = value
        del self.search_target

    @volatile_property
    def search_target(self):
        return get_reference_target(self._reference)

    # tags search target

    @property
    def tags_search_target(self):
        return self._tags_search_target

    @tags_search_target.setter
    def tags_search_target(self, value):
        self._tags_search_target = value
        del self.tags_target

    @volatile_property
    def tags_target(self):
        if self.enable_tags_search:
            return get_reference_target(self._tags_search_target)
        return None

    # collections search target

    @property
    def collections_search_target(self):
        return self._collections_search_target

    @collections_search_target.setter
    def collections_search_target(self, value):
        self._collections_search_target = value
        del self.collections_target

    @volatile_property
    def collections_target(self):
        if self.enable_collections_search:
            return get_reference_target(self._collections_search_target)
        return None

    @property
    def references(self):
        return list(filter(lambda x: x is not None, [self.reference,
                                                     self.tags_search_target,
                                                     self.collections_search_target]))

    use_references_for_views = False

    def get_targets(self, state=None):
        for reference in [self.reference, self.tags_search_target, self.collections_search_target]:
            if reference is not None:
                yield get_reference_target(reference, state)


@adapter_config(required=ISiteRoot,
                provides=ISearchManagerInfo)
def site_root_search_manager_info(context):
    """Site root search manager adapter"""
    return get_annotation_adapter(context, SEARCH_MANAGER_INFO_KEY, ISearchManagerInfo)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.thesaurus module

This module provides a custom thesaurus extension which allows to assign an
illustration on any thesaurus term.
"""

from pyams_content.component.illustration.interfaces import IBaseIllustrationTarget
from pyams_thesaurus.interfaces.extension import IThesaurusTermExtension
from pyams_utils.registry import utility_config


__docformat__ = 'restructuredtext'

from pyams_content import _


@utility_config(name='illustration',
                provides=IThesaurusTermExtension)
class IllustrationThesaurusTermExtension:
    """Illustration thesaurus term extension"""

    label = _("Illustration")
    weight = 20

    icon_css_class = 'far fa-image'

    target_interface = IBaseIllustrationTarget
    target_view = 'illustration-dialog.html'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.interfaces module

"""

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import containers, contains
from zope.container.interfaces import IOrderedContainer
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, List, Set

from pyams_content.feature.renderer import IContentRenderer, IRenderedContent
from pyams_i18n.schema import I18nTextLineField


__docformat__ = 'restructuredtext'

from pyams_content import _


PARAGRAPH_CONTAINER_KEY = 'pyams_content.paragraph'


class IBaseParagraph(IRenderedContent, IAttributeAnnotatable):
    """Base paragraph interface"""

    containers('.IParagraphContainer')

    factory_name = Attribute("Paragraph factory name")
    factory_label = Attribute("Paragraph factory label")

    icon_class = Attribute("Icon class in paragraphs list")
    icon_hint = Attribute("Icon hint in paragraphs list")

    secondary = Attribute("Boolean class attribute used to specify if this factory is secondary "
                          "or not; secondary factories are displayed in a separate sub-menu")

    visible = Bool(title=_("Visible?"),
                   description=_("Is this paragraph visible in front-office?"),
                   required=True,
                   default=True)

    anchor = Bool(title=_("Anchor?"),
                  description=_("Is this paragraph a navigation anchor?"),
                  required=True,
                  default=False)

    locked = Bool(title=_("Locked?"),
                  description=_("A locked paragraph can only be removed by a content manager "
                                "or a webmaster"),
                  required=True,
                  default=False)

    title = I18nTextLineField(title=_("§ Title"),
                              required=False)


PARAGRAPH_HIDDEN_FIELDS = ('__parent__', '__name__', 'visible', 'anchor', 'locked', 'renderer')


class IParagraphTitle(Interface):
    """Paragraph title adapter"""


class IParagraphContainer(IOrderedContainer):
    """Paragraphs container"""

    contains(IBaseParagraph)

    def append(self, value):
        """Add given value to container"""

    def get_paragraphs(self, factories):
        """Get paragraphs matching given factories"""

    def get_visible_paragraphs(self, names=None, anchors_only=False, exclude_anchors=False,
                               factories=None, limit=None):
        """Get visible paragraphs matching given arguments"""


CONTENT_PARAGRAPHS_VOCABULARY = 'pyams_content.paragraphs'


class IParagraphContainerTarget(IAttributeAnnotatable):
    """Paragraphs container marker interface"""


PARAGRAPH_FACTORIES_VOCABULARY = 'pyams_content.paragraph.factories'


#
# Paragraph factory settings
#

PARAGRAPH_FACTORY_SETTINGS_KEY = 'pyams_content.paragraph.settings'


class IParagraphFactorySettings(Interface):
    """Paragraph factory settings interface

    This interface is used to defined default auto-created paragraphs
    for a given shared tool."""

    allowed_paragraphs = Set(title=_("Allowed paragraphs"),
                             description=_("List of paragraphs allowed for this content type; if selection is empty, "
                                           "all paragraphs types will be allowed"),
                             required=False,
                             value_type=Choice(vocabulary=PARAGRAPH_FACTORIES_VOCABULARY))

    auto_created_paragraphs = List(title=_("Default paragraphs types"),
                                   description=_("Empty paragraphs of these types will be added "
                                                 "automatically to new contents of this content "
                                                 "type; if paragraphs are associated to content type, these will "
                                                 "be used instead of these ones"),
                                   required=False,
                                   value_type=Choice(vocabulary=PARAGRAPH_FACTORIES_VOCABULARY))


class IParagraphFactorySettingsTarget(Interface):
    """Paragraph factory settings target interface"""


#
# Paragraphs renderers
#

DEFAULT_PARAGRAPH_RENDERER_NAME = 'default'


class IParagraphRenderer(IContentRenderer):
    """Paragraph renderer interface"""


class ParagraphRendererChoice(Choice):
    """Paragraph renderer choice schema field factory"""

    def __init__(self, description, renderers):
        super().__init__(title=_("Renderer"),
                         description=description,
                         vocabulary=renderers,
                         default=DEFAULT_PARAGRAPH_RENDERER_NAME)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common module

"""

from hypatia.interfaces import ICatalog
from persistent import Persistent
from pyramid.events import subscriber
from pyramid.settings import asbool
from zope.container.contained import Contained
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import implementer
from zope.intid import IIntIds
from zope.lifecycleevent import IObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.interfaces import IBaseContentInfo, IObjectType
from pyams_content.shared.common.interfaces import CONTENT_TYPES_VOCABULARY, IBaseSharedTool, \
    ISharedContent, IWfSharedContent, IWfSharedContentRoles, SHARED_CONTENT_TYPES_VOCABULARY, \
    VIEWS_SHARED_CONTENT_TYPES_VOCABULARY
from pyams_i18n.content import I18nManagerMixin
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces import IDefaultProtectionPolicy
from pyams_security.interfaces.base import VIEW_PERMISSION
from pyams_security.security import ProtectedObjectMixin
from pyams_security.utility import get_principal
from pyams_sequence.interfaces import ISequentialIdInfo, ISequentialIdTarget
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.factory import get_all_factories, get_interface_base_name, get_object_factory
from pyams_utils.interfaces.text import IHTMLRenderer
from pyams_utils.property import ClassPropertyType, classproperty
from pyams_utils.registry import query_utility
from pyams_utils.request import check_request, query_request
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyams_utils.zodb import volatile_property
from pyams_workflow.interfaces import IObjectClonedEvent, IWorkflow, IWorkflowPublicationSupport, \
    IWorkflowTransitionEvent, IWorkflowVersions
from pyams_zmi.interfaces import IObjectLabel

__docformat__ = 'restructuredtext'

from pyams_content import _


@vocabulary_config(name=SHARED_CONTENT_TYPES_VOCABULARY)
class ContentTypesVocabulary(SimpleVocabulary):
    """Content types vocabulary"""

    def __init__(self, context):
        request = check_request()
        settings = request.registry.settings
        translate = request.localizer.translate
        terms = sorted([
            SimpleTerm(factory.content_type, title=translate(factory.content_name))
            for _name, factory in get_all_factories(ISharedContent)
            if asbool(settings.get(f'pyams_content.register.{factory.content_type}', True))
        ], key=lambda x: x.title)
        super().__init__(terms)


@vocabulary_config(name=VIEWS_SHARED_CONTENT_TYPES_VOCABULARY)
class ViewsContentTypesVocabulary(SimpleVocabulary):
    """Views and search folders content types vocabulary"""

    def __init__(self, context):
        request = check_request()
        settings = request.registry.settings
        translate = request.localizer.translate
        terms = sorted([
            SimpleTerm(factory.content_type, title=translate(factory.content_name))
            for _name, factory in get_all_factories(ISharedContent)
            if factory.content_view and
               asbool(settings.get(f'pyams_content.register.{factory.content_type}', True))
        ], key=lambda x: x.title)
        super().__init__(terms)


#
# Workflow shared content class and adapters
#

@implementer(IDefaultProtectionPolicy, IWfSharedContent, IWorkflowPublicationSupport)
class WfSharedContent(ProtectedObjectMixin, I18nManagerMixin, Persistent, Contained):
    """Shared data content class"""

    content_type = None
    content_name = None
    content_intf = None
    content_view = True

    handle_short_name = False
    handle_content_url = True
    handle_header = True
    handle_description = True

    title = FieldProperty(IWfSharedContent['title'])
    short_name = FieldProperty(IWfSharedContent['short_name'])
    content_url = FieldProperty(IWfSharedContent['content_url'])
    creator = FieldProperty(IWfSharedContent['creator'])
    modifiers = FieldProperty(IWfSharedContent['modifiers'])
    last_modifier = FieldProperty(IWfSharedContent['last_modifier'])
    header = FieldProperty(IWfSharedContent['header'])
    description = FieldProperty(IWfSharedContent['description'])
    keywords = FieldProperty(IWfSharedContent['keywords'])
    notepad = FieldProperty(IWfSharedContent['notepad'])

    @property
    def first_owner(self):
        """First owner getter"""
        versions = IWorkflowVersions(self, None)
        if versions is not None:
            return versions.get_version(1).creator

    @property
    def creation_label(self):
        """Creation label getter"""
        request = check_request()
        translate = request.localizer.translate
        return translate(_('{date} by {principal}')).format(
            date=format_datetime(tztime(IBaseContentInfo(self).created_date)),
            principal=get_principal(request, self.creator).title)

    @property
    def last_update_label(self):
        """Last update label getter"""
        request = check_request()
        translate = request.localizer.translate
        return translate(_('{date} by {principal}')).format(
            date=format_datetime(tztime(IBaseContentInfo(self).modified_date)),
            principal=get_principal(request, self.last_modifier).title)


@subscriber(IObjectModifiedEvent, context_selector=IWfSharedContent)
def handle_modified_shared_content(event):
    """Define content's modifiers when content is modified"""
    request = query_request()
    if request is not None:
        content = event.object
        try:
            principal_id = request.principal.id
        except AttributeError:
            pass
        else:
            modifiers = content.modifiers or set()
            if principal_id not in modifiers:
                modifiers.add(principal_id)
                content.modifiers = modifiers
                catalog = query_utility(ICatalog)
                intids = query_utility(IIntIds)
                catalog['modifiers'].reindex_doc(intids.register(content), content)
            content.last_modifier = principal_id


@subscriber(IObjectClonedEvent, context_selector=IWfSharedContent)
def handle_cloned_shared_content(event):
    """Handle cloned object when a new version is created

    Current principal is set as version creator, and is added to version
    contributors if he is not the original content's owner
    """
    request = query_request()
    principal_id = request.principal.id
    content = event.object
    content.creator = principal_id
    roles = IWfSharedContentRoles(content)
    if principal_id not in roles.owner:
        # creation of new versions doesn't change owner
        # but new creators are added to contributors list
        contributors = roles.contributors or set()
        contributors.add(principal_id)
        roles.contributors = contributors
    # reset modifiers
    content.modifiers = set()


@adapter_config(required=IWfSharedContent,
                provides=ISequentialIdInfo)
def wf_shared_content_sequence_adapter(context):
    """Shared content sequence adapter"""
    parent = get_parent(context, ISharedContent)
    if parent is not None:
        return ISequentialIdInfo(parent)


@adapter_config(required=IWfSharedContent,
                provides=IObjectType)
def wf_shared_content_object_type(context):
    """Shared content object type adapter"""
    return get_interface_base_name(context.content_intf)


@adapter_config(required=(IWfSharedContent, IPyAMSLayer),
                provides=IObjectLabel)
def wf_shared_content_label(context, request):
    """Shared content label adapter"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=IWfSharedContent,
                provides=IBaseContentInfo)
class WfSharedContentInfoAdapter(ContextAdapter):
    """Shared content base info adapter"""

    @property
    def created_date(self):
        """Creation date getter"""
        return IZopeDublinCore(self.context).created

    @property
    def modified_date(self):
        """Last modification date getter"""
        return IZopeDublinCore(self.context).modified


@adapter_config(required=IWfSharedContent,
                provides=IWorkflow)
def wf_shared_content_workflow_adapter(context):
    """Shared content workflow adapter"""
    parent = get_parent(context, IBaseSharedTool)
    return query_utility(IWorkflow, name=parent.shared_content_workflow)


#
# Main shared content class and adapters
#

@implementer(ISharedContent, ISequentialIdTarget)
class SharedContent(Persistent, Contained, metaclass=ClassPropertyType):
    """Workflow managed shared data"""

    view_permission = VIEW_PERMISSION

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''

    content_type = None
    content_name = None
    content_view = True

    @classproperty
    def content_factory(cls):
        """Content class getter"""
        return get_object_factory(IWfSharedContent, name=cls.content_type)

    @property
    def workflow_name(self):
        """Workflow getter"""
        return get_parent(self, IBaseSharedTool).shared_content_workflow

    @volatile_property
    def visible_version(self):
        workflow = IWorkflow(self)
        versions = IWorkflowVersions(self).get_versions(workflow.visible_states, sort=True)
        if versions:
            return versions[-1]
        return None


@adapter_config(required=ISharedContent,
                provides=IBaseContentInfo)
class SharedContentInfoAdapter(ContextAdapter):
    """Shared content base info adapter"""

    @property
    def created_date(self):
        """Creation date getter"""
        return IZopeDublinCore(self.context).created

    @property
    def modified_date(self):
        """Modification date getter"""
        return IZopeDublinCore(self.context).modified


@adapter_config(required=ISharedContent,
                provides=IWorkflow)
def shared_content_workflow_adapter(context):
    """Shared content workflow adapter"""
    parent = get_parent(context, IBaseSharedTool)
    return query_utility(IWorkflow, name=parent.shared_content_workflow)


@subscriber(IWorkflowTransitionEvent)
def handle_workflow_event(event):
    """Reset target on workflow transition"""
    content = get_parent(event.object, ISharedContent)
    if content is not None:
        del content.visible_version


@adapter_config(name='text_with_oid',
                required=(ISharedContent, str),
                provides=IHTMLRenderer)
@adapter_config(name='text_with_oid',
                required=(IWfSharedContent, str),
                provides=IHTMLRenderer)
class SharedContentTextWithOIDRenderer:
    """Shared content text with OID renderer"""

    def __init__(self, context, value):
        self.context = context
        self.value = value

    def render(self):
        sequence = ISequentialIdInfo(self.context, None)
        if sequence is None:
            return self.value
        return f'{self.value} ({sequence.public_oid})'



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.alert module

"""
from itertools import chain

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.feature.alert.interfaces import ALERT_MANAGER_KEY, IAlertManagerInfo
from pyams_content.shared.alert import ALERT_CONTENT_TYPE
from pyams_content.shared.alert.interfaces import IAlertManager
from pyams_sequence.reference import InternalReferenceMixin, get_reference_target
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.list import unique_iter
from pyams_utils.registry import query_utility


@factory_config(IAlertManagerInfo)
class AlertManagerInfo(InternalReferenceMixin, Persistent, Contained):
    """Alert manager info"""

    _reference = FieldProperty(IAlertManagerInfo['reference'])
    context_view = FieldProperty(IAlertManagerInfo['context_view'])

    @property
    def reference(self):
        """Internal reference getter"""
        return self._reference

    @reference.setter
    def reference(self, value):
        """Internal reference setter"""
        self._reference = value
        del self.target

    def get_visible_alerts(self, request):
        """Iterator over visible alerts"""

        def check_alert(alert):
            """Hide alerts matching current request context"""
            for target in alert.get_targets():
                if target is request.context:
                    return False
            return True

        view = get_reference_target(self.reference)
        if view is None:
            return
        yield from filter(check_alert,
                          view.get_results(context=request.context,
                                           request=request,
                                           content_type=ALERT_CONTENT_TYPE))

    def get_context_alerts(self, request, context=None):
        """Iterator over visible alerts associated with provided context"""
        if context is None:
            context = request.context
        # extract alerts from selected context view
        view = get_reference_target(self.context_view)
        if view is not None:
            shared_alerts = view.get_results(context=context,
                                             request=request,
                                             content_type=ALERT_CONTENT_TYPE)
        else:
            shared_alerts = ()
        # extract context alerts from alerts manager
        manager = query_utility(IAlertManager)
        if manager is not None:
            context_alerts = manager.find_context_alerts(context, request)
        else:
            context_alerts = ()
        yield from unique_iter(chain(shared_alerts, context_alerts))


@adapter_config(context=ISiteRoot,
                provides=IAlertManagerInfo)
def site_root_alerts_manager_info(context):
    """Site root alerts manager information factory"""
    return get_annotation_adapter(context, ALERT_MANAGER_KEY, IAlertManagerInfo)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.search.portlet.skin.interfaces module

"""

from collections import OrderedDict
from enum import Enum

from zope.contentprovider.interfaces import IContentProvider
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, Int
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_i18n.schema import I18nTextLineField
from pyams_skin.schema import BootstrapThumbnailsSelectionField

__docformat__ = 'restructuredtext'

from pyams_content import _


SEARCH_RESULTS_RENDERER_SETTINGS_KEY = 'pyams_content.search.renderer::search-results'


class HEADER_DISPLAY_MODE(Enum):
    """Header display modes"""
    FULL = 'full'
    START = 'start'
    HIDDEN = 'none'


HEADER_DISPLAY_MODES_NAMES = OrderedDict((
    (HEADER_DISPLAY_MODE.FULL, _("Display full header")),
    (HEADER_DISPLAY_MODE.START, _("Display only header start")),
    (HEADER_DISPLAY_MODE.HIDDEN, _("Hide header"))
), )


HEADER_DISPLAY_MODES_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in HEADER_DISPLAY_MODES_NAMES.items()
])


class ISearchResultsPortletBaseRendererSettings(Interface):
    """Search results portlet renderer base settings interface"""

    display_if_empty = Bool(title=_("Display if empty?"),
                            description=_("If 'no', and if no result is found, the portlet "
                                          "will not display anything"),
                            required=True,
                            default=True)

    display_results_count = Bool(title=_("Display results count?"),
                                 description=_("If 'no', results count will not be displayed"),
                                 required=True,
                                 default=True)

    allow_sorting = Bool(title=_("Allow results sorting?"),
                         description=_("If 'no', results will not be sortable"),
                         required=True,
                         default=True)

    allow_pagination = Bool(title=_("Allow pagination?"),
                            description=_("If 'no', results will not be paginated"),
                            required=True,
                            default=True)

    header_display_mode = Choice(title=_("Header display mode"),
                                 description=_("Defines how results headers will be rendered"),
                                 required=True,
                                 vocabulary=HEADER_DISPLAY_MODES_VOCABULARY,
                                 default=HEADER_DISPLAY_MODE.FULL.value)

    start_length = Int(title=_("Start length"),
                       description=_("If you choose to display only header start, you can "
                                     "specify maximum text length"),
                       required=True,
                       default=120)

    display_illustrations = Bool(title=_("Display illustrations?"),
                                 description=_("If 'no', view contents will not display "
                                               "illustrations"),
                                 required=True,
                                 default=True)

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display images thumbnails"),
        default_selection='pano',
        change_selection=True,
        default_width=3,
        change_width=True,
        required=False)


class ISearchResultsPortletDefaultRendererSettings(ISearchResultsPortletBaseRendererSettings):
    """Search results portlet default renderer settings interface"""


class ISearchResultsPortletPanelsRendererSettings(ISearchResultsPortletBaseRendererSettings):
    """Search results portlet panels renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display images thumbnails"),
        default_selection='pano',
        change_selection=True,
        default_width=12,
        change_width=False,
        required=False)

    button_title = I18nTextLineField(title=_("Button's title"),
                                     description=_("Optional navigation button's title"),
                                     required=False)


class ISearchResultsPortletCardsRendererSettings(ISearchResultsPortletBaseRendererSettings):
    """Search results portlet cards renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display images thumbnails"),
        default_selection='pano',
        change_selection=True,
        default_width=12,
        change_width=False,
        required=False)

    button_title = I18nTextLineField(title=_("Button's title"),
                                     description=_("Optional navigation button's title"),
                                     required=False)


class ISearchResultsPortletMasonryCardsRendererSettings(ISearchResultsPortletCardsRendererSettings):
    """Search results portlet Masonry cards renderer settings interface"""


#
# Search results renderers interfaces
#

class ISearchResultTitle(Interface):
    """Search result title interface"""


class ISearchResultHeader(Interface):
    """Search result header interface"""


class ISearchResultURL(Interface):
    """Search result target URL interface"""


class ISearchResultRenderer(IContentProvider):
    """Search result renderer interface"""

    title = Attribute("Search result title")
    header = Attribute("Search result header")
    url = Attribute("Search result URL")



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyramid.decorator import reify
from zope.contentprovider.interfaces import IContentProvider
from zope.interface import Interface

from pyams_content.component.thesaurus import ITagsInfo
from pyams_content.feature.search.interfaces import ISearchManagerInfo
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.tales import ITALESExtension
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config


@contentprovider_config(name='pyams_content.tags',
                        layer=IPyAMSLayer, view=Interface)
@template_config(template='templates/tags.pt', layer=IPyAMSLayer)
class TagsContentProvider(ViewContentProvider):
    """Tags content provider"""

    tags_info = None

    def update(self):
        super().update()
        self.tags_info = ITagsInfo(self.context, None)

    def render(self, template_name=''):
        if self.tags_info is None:
            return ''
        return super().render(template_name)

    @reify
    def search_target(self):
        manager = ISearchManagerInfo(self.request.root, None)
        if manager is not None:
            return manager.tags_target
        return None

    @property
    def tags(self):
        tags = self.tags_info.tags or ()
        yield from sorted(tags, key=lambda x: (x.order or 999, x.alt or x.label))


@adapter_config(name='tags',
                required=(Interface, Interface, Interface),
                provides=ITALESExtension)
class TagsTALESExtension(ContextRequestViewAdapter):
    """tales:tags(context) TALES extension"""

    def render(self, context=None):
        if context is None:
            context = self.context
        provider = self.request.registry.queryMultiAdapter((context, self.request, self.view),
                                                           IContentProvider,
                                                           name='pyams_content.tags')
        if provider is None:
            return ''
        provider.update()
        return provider.render()



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.component.association import IAssociationContainer, IAssociationContainerTarget
from pyams_content.component.links import EXTERNAL_LINK_ICON_CLASS, IExternalLink, IInternalLink, \
    IMailtoLink, INTERNAL_LINK_ICON_CLASS, MAILTO_LINK_ICON_CLASS
from pyams_content.component.paragraph.zmi import IParagraphContainerFullTable
from pyams_content.component.paragraph.zmi.container import ParagraphTitleToolbarItemMixin
from pyams_content.component.paragraph.zmi.interfaces import IParagraphTitleToolbar
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='internal-links',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=20)
class InternalLinksTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """Internal links title toolbar viewlet"""

    icon_class = INTERNAL_LINK_ICON_CLASS
    icon_hint = _("Internal links")

    target_intf = IAssociationContainer
    item_intf = IInternalLink


@viewlet_config(name='external-links',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=20)
class ExternalLinksTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """External links title toolbar viewlet"""

    icon_class = EXTERNAL_LINK_ICON_CLASS
    icon_hint = _("External links")

    target_intf = IAssociationContainer
    item_intf = IExternalLink


@viewlet_config(name='mailto-links',
                context=IAssociationContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=30)
class MailtoLinksTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """Mailto links title toolbar viewlet"""

    icon_class = MAILTO_LINK_ICON_CLASS
    icon_hint = _("Mailto links")

    target_intf = IAssociationContainer
    item_intf = IMailtoLink



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.review.manager module

Review target adapter.
"""

from pyramid.renderers import render

from pyams_content.feature.review import IReviewComments, IReviewManager, IReviewTarget, \
    ReviewComment
from pyams_content.shared.common.interfaces import IWfSharedContentRoles
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_mail.message import HTMLMessage
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.notification import INotificationSettings
from pyams_security.principal import MissingPrincipal
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=IReviewTarget,
                provides=IReviewManager)
class ReviewManagerAdapter(ContextAdapter):
    """Review manager adapter"""

    def ask_review(self, reviewers, comment, notify_all=True, request=None):
        """Ask for content review"""
        roles = IWfSharedContentRoles(self.context, None)
        if roles is None:
            return
        # check request
        if request is None:
            request = check_request()
        translate = request.localizer.translate
        # initialize mailer
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        settings = INotificationSettings(sm)
        sender_name = request.principal.title \
            if request.principal is not None else settings.sender_name
        sender_address = settings.sender_email
        sender = sm.get_raw_principal(request.principal.id)
        sender_mail_info = IPrincipalMailInfo(sender, None)
        if sender_mail_info is not None:
            for sender_name, sender_address in sender_mail_info.get_addresses():
                break
        if settings.enable_notifications:
            mailer = settings.get_mailer()
        else:
            mailer = None
        # create message
        html_body = render('zmi/templates/mail-notification.pt', request=request, value={
            'settings': settings,
            'comment': comment,
            'sender': sender_name
        })
        # notify reviewers
        notifications = 0
        readers = roles.readers.copy()
        for reviewer in reviewers:
            if settings.enable_notifications and \
                    (mailer is not None) and \
                    (notify_all or (reviewer not in readers)):
                principal = sm.get_raw_principal(reviewer)
                if not isinstance(principal, MissingPrincipal):
                    mail_info = IPrincipalMailInfo(principal, None)
                    if mail_info is not None:
                        for name, address in mail_info.get_addresses():
                            message = HTMLMessage(
                                subject=translate(_("{service_name}A content review is "
                                                    "requested")).format(
                                    service_name=f'[{settings.subject_prefix}] '
                                        if settings.subject_prefix else ''),
                                from_addr=f'{settings.sender_name} <{settings.sender_email}>',
                                reply_to=f'{sender_name} <{sender_address}>',
                                to_addr=f'{name} <{address}>',
                                html=html_body)
                            mailer.send(message)
                            notifications += 1
            readers.add(reviewer)
        roles.readers = readers
        # add comment
        review_comment = ReviewComment(owner=request.principal.id,
                                       comment_type='request',
                                       comment=translate(_("Request comment: "
                                                           "{comment}")).format(comment=comment),
                                       reviewers=reviewers)
        IReviewComments(self.context).add_comment(review_comment)
        # return notifications count
        return notifications



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation.zmi module

This module defines navigation management components interfaces.
"""

from zope.interface import Interface

from pyams_content.component.association.zmi import IAssociationsTable
from pyams_content.component.association.zmi.interfaces import IAssociationsContainerEditForm, \
    IAssociationsView


__docformat__ = 'restructuredtext'


class IMenuLinksView(IAssociationsView):
    """Menu links view marker interface"""


class IMenuLinksTable(IAssociationsTable):
    """Menu links table marker interface"""


class IMenuLinksContainerEditForm(IAssociationsContainerEditForm):
    """Menu links container edit form marker interface"""


class IMenusView(Interface):
    """Menus view marker interface"""


class IMenusTable(Interface):
    """Menus table marker interface"""


class IMenusContainerEditForm(Interface):
    """Menus container edit form marker interface"""


class IMenuAddForm(Interface):
    """Menu add form marker interface"""


class IMenuEditForm(Interface):
    """Menu edit form marker interface"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.widget.interfaces module

"""

__docformat__ = 'restructuredtext'

from zope.interface import Attribute

from pyams_form.interfaces import INPUT_MODE
from pyams_form.interfaces.widget import ITextWidget
from pyams_form.template import widget_template_config
from pyams_zmi.interfaces import IAdminLayer


@widget_template_config(mode=INPUT_MODE,
                        template='templates/folder-select-input.pt',
                        layer=IAdminLayer)
class ISiteManagerFoldersSelectorWidget(ITextWidget):
    """Site manager folders selector widget interface"""

    permission = Attribute("Permission required to select a given node")



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.types module

This module define components which are used for management of typed
shared contents.
"""

__docformat__ = 'restructuredtext'

from pyramid.decorator import reify
from zope.interface import Interface

from pyams_content.shared.common.interfaces import ISharedTool
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_content.shared.common.zmi import SharedContentPropertiesEditForm
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer


class TypedSharedContentPropertiesEditForm(SharedContentPropertiesEditForm):
    """Typed shared content properties edit form"""

    interface = IWfTypedSharedContent
    fieldnames = ('title', 'short_name', 'content_url', 'data_type',
                  'header', 'description', 'notepad')


class TypedSharedContentCustomInfoEditForm(AdminEditForm):
    """Typed shared content custom information edit form"""

    @reify
    def datatype(self):
        """Form context datatype getter"""
        context = self.context
        if IWfTypedSharedContent.providedBy(context):
            return context.get_data_type()
        return None

    @property
    def fields(self):
        """Form fields getter"""
        datatype = self.datatype
        if datatype is None:
            return Fields(Interface)
        manager = get_utility(ISharedTool, name=self.context.content_type)
        return Fields(manager.shared_content_info_factory).select(*datatype.field_names)


@adapter_config(required=(IWfTypedSharedContent, IAdminLayer, TypedSharedContentCustomInfoEditForm),
                provides=IFormContent)
def typed_shared_content_custom_info_edit_form_content(context, request, layer):
    """Typed shared content custom info edit content getter"""
    manager = get_utility(ISharedTool, name=context.content_type)
    return manager.shared_content_info_factory(context)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.component.gallery.interfaces import IGalleryContainer
from pyams_content.component.gallery.zmi.interfaces import IGalleryMediasView, IGalleryMediaThumbnailView
from pyams_utils.factory import create_object
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'


def get_json_gallery_refresh_event(context, request, view):
    """Get gallery refresh event"""
    gallery = get_parent(context, IGalleryContainer)
    provider = create_object(IGalleryMediasView,
                             context=gallery, request=request, view=view)
    if provider is not None:
        provider.update()
        return {
            'callback': 'MyAMS.helpers.refreshElement',
            'options': {
                'object_id': f'gallery_{ICacheKeyValue(gallery)}',
                'content': provider.render()
            }
        }
    return None


def get_json_gallery_media_refresh_event(context, request, view):
    """Get gallery media refresh event"""
    provider = create_object(IGalleryMediaThumbnailView,
                             context=context, request=request, view=view)
    if provider is not None:
        provider.update()
        return {
            'callback': 'MyAMS.helpers.refreshElement',
            'options': {
                'object_id': f'media_{ICacheKeyValue(context)}',
                'content': provider.render()
            }
        }
    return None



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.form.skin module

Form help viewlet.
"""

__docformat__ = 'restructuredtext'

from pyams_content.shared.form import IWfForm
from pyams_content.skin.interfaces import IContentTitle
from pyams_fields.skin import IFormFieldContainerInputForm
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config


@adapter_config(required=(IWfForm, IPyAMSUserLayer),
                provides=IContentTitle)
def form_title(context, request):
    """Custom form title getter"""
    i18n = II18n(context)
    return i18n.query_attribute('alt_title', request=request) or \
        i18n.query_attribute('title', request=request)


@viewlet_config(name='form-header.help',
                context=IWfForm, layer=IPyAMSUserLayer, view=IFormFieldContainerInputForm,
                manager=IFormHeaderViewletManager, weight=10)
class FormInputHelpMessage(AlertMessage):
    """Form input form help"""

    def __new__(cls, context, request, view, manager):
        header = II18n(context).query_attribute('form_header', request=request)
        if not header:
            return None
        return AlertMessage.__new__(cls)

    @property
    def message(self):
        """Message getter"""
        return II18n(self.context).query_attribute('form_header', request=self.request)

    message_renderer = 'raw'



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.portlet.zmi.title module

This module defines management components of shared contents title portlet.
"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_content.shared.common.portlet.interfaces import ISharedContentTitlePortletSettings
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortletPreviewer
from pyams_portal.zmi import PortletPreviewer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config


@adapter_config(required=(Interface, IPyAMSLayer, Interface, ISharedContentTitlePortletSettings),
                provides=IPortletPreviewer)
@template_config(template='templates/title-preview.pt', layer=IPyAMSLayer)
class SharedContentTitlePortletPreviewer(PortletPreviewer):
    """Shared content title portlet previewer"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.root.configuration module

"""

import logging

from persistent import Persistent
from pyramid.path import DottedNameResolver
from zope.container.contained import Contained
from zope.lifecycleevent import ObjectCreatedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.reference import IReferenceManager
from pyams_content.root import ISiteRootToolsConfiguration, SITE_ROOT_TOOLS_CONFIGURATION_KEY
from pyams_content.shared.common.interfaces import ISharedToolContainer
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config, get_object_factory
from pyams_utils.interfaces import MISSING_INFO

__docformat__ = 'restructuredtext'


LOGGER = logging.getLogger('PyAMS (content)')


@factory_config(ISiteRootToolsConfiguration)
class SiteRootToolsConfiguration(Persistent, Contained):
    """Site root tools configuration"""

    tables_manager_name = FieldProperty(ISiteRootToolsConfiguration['tables_manager_name'])
    tables_names = FieldProperty(ISiteRootToolsConfiguration['tables_names'])

    tools_manager_name = FieldProperty(ISiteRootToolsConfiguration['tools_manager_name'])
    tools_names = FieldProperty(ISiteRootToolsConfiguration['tools_names'])

    def get_tables_manager(self):
        """Tables manager getter"""
        name = self.tables_manager_name
        if name is not None:
            return self.__parent__.get(self.tables_manager_name)
        return None

    def check_tables_manager(self, registry):
        """Tables manager checker"""
        manager = self.get_tables_manager()
        if manager is None:
            settings = registry.settings
            factory = settings.get('pyams_content.config.references_manager_factory')
            if factory is not None:
                factory = DottedNameResolver().resolve(factory)
            if factory is None:
                factory = get_object_factory(IReferenceManager)
            if factory is not None:
                manager = factory()
            if manager is not None:
                self.tables_manager_name = name = \
                    settings.get('pyams_content.config.references_manager_name', 'references')
                self.__parent__[name] = manager
                self.tables_names = {}
        return manager

    def check_table(self, interface, table_name, registry):
        """Reference table checker"""
        table = None
        manager = self.check_tables_manager(registry)
        name = self.tables_names.get(interface)
        if name is not None:
            table = manager.get(name)
            if table is not None:
                return table
        factory = registry.settings.get(f'pyams_content.config.{table_name}_table_factory')
        if factory:
            if factory.lower in (MISSING_INFO, 'off', 'none', 'disabled'):
                return None
            factory = DottedNameResolver().resolve(factory)
        if factory is None:
            factory = get_object_factory(interface)
        if factory is not None:
            LOGGER.info(f'Creating table from factory {factory!r}...')
            table = factory()
            registry.notify(ObjectCreatedEvent(table))
            name = registry.settings.get(f'pyams_content.config.{table_name}_table_name',
                                         table_name)
            manager[name] = table
            self.tables_names[interface] = name
        return table

    def get_tools_manager(self):
        """Tools manager getter"""
        name = self.tools_manager_name
        if name is not None:
            return self.__parent__.get(self.tools_manager_name)
        return None

    def check_tools_manager(self, registry):
        """Tools manager checker"""
        manager = self.get_tools_manager()
        if manager is None:
            settings = registry.settings
            factory = settings.get('pyams_content.config.tools_manager_factory')
            if factory is not None:
                factory = DottedNameResolver().resolve(factory)
            if factory is None:
                factory = get_object_factory(ISharedToolContainer)
            if factory is not None:
                manager = factory()
            if manager is not None:
                self.tools_manager_name = name = \
                    settings.get('pyams_content.config.tools_manager_name', 'tools')
                self.__parent__[name] = manager
                self.tools_names = {}
        return manager

    def check_tool(self, interface, tool_name, registry):
        """Shared tool checker"""
        tool = None
        manager = self.check_tools_manager(registry)
        name = self.tools_names.get(interface)
        if name is not None:
            tool = manager.get(name)
            if tool is not None:
                return tool
        name = registry.settings.get(f'pyams_content.config.{tool_name}_tool_name',
                                     tool_name)
        if name in manager:
            tool = manager.get(name)
            if interface.providedBy(tool):
                self.tools_names[interface] = name
                return tool
        factory = registry.settings.get(f'pyams_content.config.{tool_name}_factory')
        if factory:
            if factory.lower in (MISSING_INFO, 'off', 'none', 'disabled'):
                return None
        if factory is not None:
            factory = DottedNameResolver().resolve(factory)
        if factory is None:
            factory = get_object_factory(interface)
        if factory is not None:
            LOGGER.info(f'Creating shared tool from factory {factory!r}...')
            tool = factory()
            registry.notify(ObjectCreatedEvent(tool))
            manager[name] = tool
            self.tools_names[interface] = name
        return tool


@adapter_config(required=ISiteRoot,
                provides=ISiteRootToolsConfiguration)
def site_root_tools_configuration_adapter(context):
    """Site root tools configuration adapter"""
    return get_annotation_adapter(context, SITE_ROOT_TOOLS_CONFIGURATION_KEY,
                                  ISiteRootToolsConfiguration)



#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.interfaces package

This module defines main content permissions, roles and interfaces.
"""

from zope.annotation import IAttributeAnnotatable
from zope.interface import Interface
from zope.location.interfaces import IContained
from zope.schema import Datetime, Set, TextLine

from pyams_i18n.schema import I18nTextLineField


__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Custom permissions
#

MANAGE_SITE_ROOT_PERMISSION = 'pyams.ManageSiteRoot'
'''Permission required to manage main site root properties'''

MANAGE_REFERENCE_TABLE_PERMISSION = 'pyams.ManageReferenceTable'
'''Permission required to manage reference table properties'''

MANAGE_SITE_TREE_PERMISSION = 'pyams.ManageSiteTree'
'''Permission required to create first level site elements'''

MANAGE_SITE_PERMISSION = 'pyams.ManageSite'
'''Permission required to manager inner site or blog properties'''

MANAGE_TOOL_PERMISSION = 'pyams.ManageTool'
'''Permission required to manager shared tool properties'''

CREATE_CONTENT_PERMISSION = 'pyams.CreateContent'
'''Permission required to create a new content'''

CREATE_VERSION_PERMISSION = 'pyams.CreateVersion'
'''Permission required to create a new version of an existing content'''

MANAGE_CONTENT_PERMISSION = 'pyams.ManageContent'
'''Permission required to manager properties of an existing content'''

COMMENT_CONTENT_PERMISSION = 'pyams.CommentContent'
'''Permission required to add comments on an existing content'''

PUBLISH_CONTENT_PERMISSION = 'pyams.PublishContent'
'''Permission required to publish or retire an existing content'''


#
# Custom roles
#

WEBMASTER_ROLE = 'pyams.Webmaster'
'''Webmaster role has all permissions on all contents'''

REFERENCE_MANAGER_ROLE = 'pyams.ReferenceManager'
'''References manager role has permissions to handle references tables'''

PILOT_ROLE = 'pyams.Pilot'
'''Pilot role is allowed to manage tools configuration and permissions'''

MANAGER_ROLE = 'pyams.Manager'
'''Manager role is allowed to manage contents workflow'''

OWNER_ROLE = 'pyams.Owner'
'''Content owner role is allowed to manage content properties until publication'''

CONTRIBUTOR_ROLE = 'pyams.Contributor'
'''Contributor role is allowed to create new contents'''

READER_ROLE = 'pyams.Reader'
'''Reader role is allowed to read and comment contents while still in draft state'''

OPERATOR_ROLE = 'pyams.Operator'
'''Operator role is allowed to access management interface'''

GUEST_ROLE = 'pyams.Guest'
'''Guest role is allowed to view contents'''


#
# Custom routes
#

OID_ACCESS_ROUTE = 'pyams_content.oid_access'
'''"/+/oid" direct access route name'''

OID_ACCESS_PATH = '/+/{oid}*view'
'''"/+/oid" direct access path'''


#
# Base content interfaces
#

class IBaseContent(IContained, IAttributeAnnotatable):
    """Base content interface"""

    __name__ = TextLine(title=_("Unique key"),
                        description=_("WARNING: this key can't be modified after creation!!! "
                                      "Spaces, uppercase letters or accentuated characters will "
                                      "be replaced automatically."),
                        required=True)

    title = I18nTextLineField(title=_("Title"),
                              description=_("Visible label used to display content"),
                              required=True)

    short_name = I18nTextLineField(title=_("Short name"),
                                   description=_("Short name used in breadcrumbs"),
                                   required=True)


class IBaseContentInfo(Interface):
    """Base content info interface"""

    created_date = Datetime(title=_("Creation date"),
                            required=False,
                            readonly=True)

    modified_date = Datetime(title=_("Modification date"),
                             required=False,
                             readonly=False)


class IObjectType(Interface):
    """Object type value interface"""


class IObjectTypes(Interface):
    """Object types """

    object_types = Set(title=_("Object types"),
                       value_type=TextLine(),
                       required=False)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_content.feature.alert import IAlertManagerInfo
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config


@contentprovider_config(name='pyams_content.alerts',
                        layer=IPyAMSUserLayer, view=Interface)
@template_config(template='templates/alerts.pt',
                 layer=IPyAMSUserLayer)
class AlertsContentProvider(ViewContentProvider):
    """Alerts content provider"""

    def get_alerts(self):
        """Alerts getter"""
        alerts = IAlertManagerInfo(self.request.root, None)
        if alerts is not None:
            yield from alerts.get_visible_alerts(self.request)

    def get_gravity(self, alert):
        """Alert gravity getter"""
        alert_type = alert.get_alert_type()
        if alert_type is not None:
            return II18n(alert_type).query_attribute('label', request=self.request)


@contentprovider_config(name='pyams_content.context_alerts',
                        layer=IPyAMSUserLayer, view=Interface)
@template_config(template='templates/context-alerts.pt',
                 layer=IPyAMSUserLayer)
class ContextAlertsContentProvider(AlertsContentProvider):
    """Content alerts content provider"""

    def get_alerts(self):
        """Context alerts getter"""
        alerts = IAlertManagerInfo(self.request.root, None)
        if alerts is not None:
            yield from alerts.get_context_alerts(self.request)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.zmi module

This module provides base paragraphs management components.
"""

from zope.interface import Interface, implementer

from pyams_content.component.association.interfaces import IAssociationContainer
from pyams_content.component.association.zmi.interfaces import IAssociationsTable
from pyams_content.component.paragraph.interfaces import IBaseParagraph, IParagraphContainer, \
    IParagraphContainerTarget, IParagraphFactorySettings, IParagraphFactorySettingsTarget, \
    PARAGRAPH_HIDDEN_FIELDS
from pyams_content.component.paragraph.zmi.helper import get_json_paragraph_toolbar_refresh_event
from pyams_content.component.paragraph.zmi.interfaces import IInnerParagraphEditForm, \
    IParagraphAddForm, IParagraphContainerBaseTable, IParagraphContainerFullTable, \
    IParagraphContainerView, IParagraphRendererSettingsEditForm
from pyams_content.feature.renderer.interfaces import IRendererSettings
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.interfaces import PREVIEW_MODE
from pyams_portal.skin.page import PortalContextPreviewPage
from pyams_portal.zmi.portlet import PortletRendererSettingsEditForm
from pyams_portal.zmi.widget import RendererSelectFieldWidget
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.security import ProtectedViewObjectMixin
from pyams_skin.schema.button import ActionButton
from pyams_skin.viewlet.menu import MenuDivider, MenuItem
from pyams_utils.adapter import ContextRequestViewAdapter, NullAdapter, adapter_config
from pyams_utils.factory import get_all_factories, get_object_factory
from pyams_utils.request import get_annotations
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import get_label, viewletmanager_config
from pyams_viewlet.viewlet import EmptyContentProvider, contentprovider_config, viewlet_config
from pyams_zmi.form import AdminEditForm, AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.event import get_json_table_refresh_callback, \
    get_json_table_row_add_callback, get_json_widget_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IEditFormButtons, IFormTitle, IPropertiesEditForm
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, IPropertiesMenu, \
    IToolbarViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_hint, get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem
from pyams_zmi.zmi.viewlet.toolbar import AddingsViewletManager

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='paragraph-types.menu',
                context=IParagraphFactorySettingsTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=400,
                permission=MANAGE_TOOL_PERMISSION)
class ParagraphFactorySettingsMenu(NavigationMenuItem):
    """Paragraph factory settings menu"""

    label = _("Paragraphs types")
    href = '#paragraph-types.html'


@ajax_form_config(name='paragraph-types.html',
                  context=IParagraphFactorySettingsTarget, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class ParagraphFactorySettingsEditForm(AdminEditForm):
    """Paragraph factory settings edit form"""

    title = _("Paragraph types")
    legend = _("Shared content paragraph types")

    fields = Fields(IParagraphFactorySettings)


@adapter_config(required=(IParagraphFactorySettingsTarget, IAdminLayer,
                          ParagraphFactorySettingsEditForm),
                provides=IFormContent)
def get_paragraph_factory_settings_edit_form_content(context, request, form):
    """Paragraph factory settings edit form content getter"""
    return IParagraphFactorySettings(context)


#
# Paragraph add menu
#

@viewletmanager_config(name='pyams.context_addings',
                       context=IParagraphContainerTarget, layer=IAdminLayer,
                       view=IParagraphContainerBaseTable, manager=IToolbarViewletManager, weight=10,
                       provides=IContextAddingsViewletManager)
class ParagraphContainerAddingsViewletManager(AddingsViewletManager):
    """Paragraph container addings viewlet manager"""

    def sort(self, viewlets):
        """Viewlets sorter"""

        def get_sort_order(viewlet):
            menu = viewlet[1]
            if isinstance(menu, MenuDivider):
                return getattr(menu, 'weight', 500), None
            factory = get_object_factory(IBaseParagraph, name=menu.factory_name)
            if factory.factory.secondary:
                return 900, get_label(viewlet, self.request)
            return getattr(menu, 'weight', 500), get_label(viewlet, self.request)

        return sorted(viewlets, key=get_sort_order)


@viewlet_config(name='paragraphs.divider',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable, manager=IContextAddingsViewletManager, weight=500)
class ParagraphAddMenuDivider(ProtectedViewObjectMixin, MenuDivider):
    """Paragraph add menu divider"""

    def __new__(cls, context, request, view, manager):
        target = get_parent(context, IParagraphFactorySettingsTarget)
        if target is None:
            return MenuDivider.__new__(cls)
        has_primary = False
        has_secondary = False
        settings = IParagraphFactorySettings(target)
        factories = settings.allowed_paragraphs or ()
        if not factories:
            factories = (name for name, factory in get_all_factories(IBaseParagraph))
        for factory_name in factories:
            factory = get_object_factory(IBaseParagraph, name=factory_name)
            if factory is None:
                continue
            if factory.factory.secondary:
                has_secondary = True
            else:
                has_primary = True
            if has_primary and has_secondary:
                return MenuDivider.__new__(cls)
        return None


class BaseParagraphAddMenu(ProtectedViewObjectMixin, MenuItem):
    """Base paragraph add menu"""

    factory_name = None
    modal_target = True

    def __new__(cls, context, request, view, manager):
        target = get_parent(context, IParagraphFactorySettingsTarget)
        if target is not None:
            allowed = IParagraphFactorySettings(target).allowed_paragraphs or ()
            if allowed and (cls.factory_name not in allowed):
                return None
        return MenuItem.__new__(cls)

    def get_href(self):
        container = IParagraphContainer(self.context)
        return absolute_url(container, self.request, self.href)


@implementer(IParagraphAddForm)
class BaseParagraphAddForm(AdminModalAddForm):
    """Base paragraph add form"""

    prefix = 'addform.'

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        factory = get_object_factory(self.content_factory)
        return translate(_("New paragraph: {}")).format(translate(factory.factory.factory_label))

    legend = _("New paragraph properties")
    modal_class = 'modal-xl'

    @property
    def fields(self):
        """Form fields getter"""
        fields = super().fields
        if fields:
            return fields
        return Fields(self.content_factory).omit(*PARAGRAPH_HIDDEN_FIELDS) + \
            Fields(self.content_factory).select('renderer')

    def add(self, obj):
        """Add paragraph to container"""
        IParagraphContainer(self.context).append(obj)


@adapter_config(required=(IParagraphContainer, IAdminLayer, IParagraphAddForm),
                provides=IFormTitle)
def paragraph_add_form_title(context, request, view):
    """Paragraph add form title"""
    parent = get_parent(context, IParagraphContainerTarget)
    return TITLE_SPAN_BREAK.format(
        get_object_hint(parent, request, view),
        get_object_label(parent, request, view))


def get_json_paragraph_editor_open_event(context, request, table_factory, item):
    """Get paragraph editor opening event"""
    factory = get_object_factory(table_factory)
    table = factory(context, request)
    return {
        'module': 'content',
        'callback': 'MyAMS.content.paragraphs.switchEditor',
        'options': {
            'object_id': table.get_row_id(item)
        }
    }


@adapter_config(required=(IParagraphContainer, IAdminLayer, IParagraphAddForm),
                provides=IAJAXFormRenderer)
class ParagraphAddFormRenderer(ContextRequestViewAdapter):
    """Paragraph add form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        target = get_parent(self.context, IParagraphContainerTarget)
        table_factory = IParagraphContainerFullTable if IWfSharedContent.providedBy(target) \
            else IParagraphContainerBaseTable
        callbacks = [
            get_json_table_row_add_callback(self.context, self.request,
                                            table_factory, changes)
        ]
        if IWfSharedContent.providedBy(target):
            callbacks.append(get_json_paragraph_editor_open_event(self.context, self.request,
                                                                  table_factory, changes))
        return {
            'status': 'success',
            'callbacks': callbacks
        }


#
# Paragraphs edit forms
#

@implementer(IPropertiesEditForm)
class ParagraphPropertiesEditFormMixin:
    """Paragraph properties edit form mixin"""

    legend = _("Paragraph properties")

    label_css_class = 'col-sm-2 col-md-3'
    input_css_class = 'col-sm-10 col-md-9'

    @property
    def prefix(self):
        """Form prefix getter"""
        return f'form_{self.context.__name__}.'

    @property
    def fields(self):
        """Form fields getter"""
        fields = super().fields
        if fields:
            return fields
        fields = Fields(self.context.factory_intf).omit(*PARAGRAPH_HIDDEN_FIELDS) + \
            Fields(self.context.factory_intf).select('renderer')
        fields['renderer'].widget_factory = RendererSelectFieldWidget
        return fields


class IInnerParagraphEditFormButtons(IEditFormButtons):
    """Inner paragraph edit form buttons interface"""

    preview = ActionButton(name='preview',
                           title=_("Preview"))


@adapter_config(required=(IBaseParagraph, IAdminLayer),
                provides=IInnerParagraphEditForm)
class InnerParagraphPropertiesEditForm(ParagraphPropertiesEditFormMixin, AdminEditForm):
    """Default inner paragraph edit form"""

    hide_section = True
    title = None

    buttons = Buttons(IInnerParagraphEditFormButtons)
    ajax_form_handler = 'properties.json'

    def update_actions(self):
        """Actions update"""
        super().update_actions()
        preview = self.actions.get('preview')
        if preview is not None:
            preview.icon_class = 'fas fa-binoculars'
            preview.icon_only = True
            preview.href = absolute_url(self.context, self.request, 'modal-preview.html')
            preview.modal_target = True
            preview.hint = self.request.localizer.translate(_("Preview"))

    @handler(IInnerParagraphEditFormButtons['apply'])
    def handle_apply(self, action):
        """Apply button handler"""
        super().handle_apply(self, action)


@adapter_config(required=(IBaseParagraph, IAdminLayer, IParagraphContainerBaseTable),
                provides=ITableElementEditor)
class BaseParagraphTableElementEditor(TableElementEditor):
    """Base paragraph table element editor"""


@adapter_config(required=(IBaseParagraph, IAdminLayer, IParagraphContainerFullTable),
                provides=ITableElementEditor)
class BaseParagraphFullTableElementEditor(NullAdapter):
    """Base paragraph full table element editor"""


@ajax_form_config(name='properties.html',
                  context=IBaseParagraph, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ParagraphPropertiesEditForm(ParagraphPropertiesEditFormMixin, AdminModalEditForm):
    """Paragraph properties edit form"""

    modal_class = 'modal-xl'


@adapter_config(name='main',
                required=(IBaseParagraph, IAdminLayer, ParagraphPropertiesEditFormMixin),
                provides=IAJAXFormRenderer)
class BaseParagraphPropertiesEditFormRenderer(ContextRequestViewAdapter):
    """Base paragraph properties edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        result = {}
        if changes:
            event = get_json_paragraph_toolbar_refresh_event(self.context, self.request)
            if event is not None:
                result.setdefault('callbacks', []).append(event)
            result.setdefault('callbacks', []).append({
                'callback': 'MyAMS.content.paragraphs.refreshTitle',
                'options': {
                    'element_name': self.context.__name__,
                    'title': get_object_label(self.context, self.request)
                }
            })
        if 'renderer' in changes.get(self.context.factory_intf, ()):
            result.setdefault('callbacks', []).append(
                get_json_widget_refresh_callback(self.view, 'renderer', self.request))
            renderer = self.context.get_renderer(self.request)
            if (renderer is not None) and (renderer.settings_interface is not None):
                translate = self.request.localizer.translate
                result['closeForm'] = False
                result['messagebox'] = {
                    'status': 'info',
                    'title': translate(_("Updated renderer")),
                    'message': translate(_("You changed renderer selection. Don't omit to "
                                           "check renderer properties...")),
                    'timeout': 5000
                }
        container = IAssociationContainer(self.context, None)
        if container is not None:
            result.setdefault('callbacks', []).append(
                get_json_table_refresh_callback(container, self.request, IAssociationsTable))
        return result


#
# Paragraph renderer properties
#

@ajax_form_config(name='renderer-settings.html',
                  context=IBaseParagraph, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
@implementer(IParagraphRendererSettingsEditForm)
class BaseParagraphRendererSettingsEditForm(PortletRendererSettingsEditForm):
    """Base paragraph renderer settings edit form"""


@adapter_config(required=(IBaseParagraph, IAdminLayer, BaseParagraphRendererSettingsEditForm),
                provides=IFormTitle)
def base_paragraph_renderer_settings_edit_form_title(context, request, form):
    """Base paragraph renderer settings edit form title"""
    container = get_parent(context, IParagraphContainerTarget)
    return TITLE_SPAN_BREAK.format(
        get_object_label(container, request, form),
        get_object_label(context, request, form))


@adapter_config(required=(IBaseParagraph, IAdminLayer, IParagraphRendererSettingsEditForm),
                provides=IFormContent)
def get_paragraph_renderer_settings_edit_form_content(context, request, form):
    """Paragraph renderer settings edit form content getter"""
    renderer = context.get_renderer(request)
    if renderer.settings_interface is None:
        return None
    return IRendererSettings(context)


#
# Paragraph preview
#

@pagelet_config(name='preview.html',
                context=IBaseParagraph, request_type=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class ParagraphPreviewPage(PortalContextPreviewPage):
    """Paragraph preview page"""

    def __init__(self, context, request):
        super().__init__(context, request)
        self.renderer = context.get_renderer(request)

    def update(self):
        """Page update"""
        get_annotations(self.request)[PREVIEW_MODE] = True
        if self.renderer is not None:
            self.renderer.update()

    def render(self):
        """Page renderer"""
        if self.renderer is not None:
            return self.renderer.render()
        return ''


@contentprovider_config(name='pyams_portal.header',
                        context=IBaseParagraph, layer=IPyAMSLayer, view=Interface)
class ParagraphHeaderContentProvider(EmptyContentProvider):
    """Paragraph header content provider"""


@contentprovider_config(name='pyams_portal.footer',
                        context=IBaseParagraph, layer=IPyAMSLayer, view=Interface)
class ParagraphFooterContentProvider(EmptyContentProvider):
    """Paragraph footer content provider"""



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.settings module

This module defines persistent classes and adapters which are used to handle
paragraphs factory settings.

These settings are used to define which types of paragraphs are allowed in
each content.
"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.paragraph.interfaces import IParagraphFactorySettings, \
    IParagraphFactorySettingsTarget, PARAGRAPH_FACTORY_SETTINGS_KEY
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config


@factory_config(IParagraphFactorySettings)
class ParagraphFactorySettings(Persistent):
    """Paragraph factory settings"""

    allowed_paragraphs = FieldProperty(IParagraphFactorySettings['allowed_paragraphs'])
    auto_created_paragraphs = FieldProperty(IParagraphFactorySettings['auto_created_paragraphs'])


@adapter_config(required=IParagraphFactorySettingsTarget,
                provides=IParagraphFactorySettings)
def paragraph_factory_settings_adapter(context):
    """Paragraph factory settings adapter"""
    return get_annotation_adapter(context, PARAGRAPH_FACTORY_SETTINGS_KEY,
                                  IParagraphFactorySettings)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.provider.dailymotion module

This module defines settings used for Dailymotion videos integration.
"""

import re

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.video import IExternalVideo, IExternalVideoProvider, IExternalVideoSettings, \
    external_video_settings
from pyams_content.component.video.provider.interfaces import IDailymotionVideoSettings
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.registry import utility_config

__docformat__ = 'restructuredtext'

from pyams_content import _


DAILYMOTION_BASE_URL = re.compile(r'https?://(?:dai\.ly|www\.dailymotion\.com/video)/([^/]+)')


@factory_config(IDailymotionVideoSettings)
class YoutubeVideoSettings(Persistent):
    """Youtube video settings"""

    _video_id = FieldProperty(IDailymotionVideoSettings['video_id'])
    start_at = FieldProperty(IDailymotionVideoSettings['start_at'])
    autoplay = FieldProperty(IDailymotionVideoSettings['autoplay'])
    show_info = FieldProperty(IDailymotionVideoSettings['show_info'])
    show_commands = FieldProperty(IDailymotionVideoSettings['show_commands'])
    ui_theme = FieldProperty(IDailymotionVideoSettings['ui_theme'])
    show_branding = FieldProperty(IDailymotionVideoSettings['show_branding'])
    show_endscreen = FieldProperty(IDailymotionVideoSettings['show_endscreen'])
    allow_fullscreen = FieldProperty(IDailymotionVideoSettings['allow_fullscreen'])
    allow_sharing = FieldProperty(IDailymotionVideoSettings['allow_sharing'])
    width = FieldProperty(IDailymotionVideoSettings['width'])
    height = FieldProperty(IDailymotionVideoSettings['height'])

    @property
    def video_id(self):
        return self._video_id

    @video_id.setter
    def video_id(self, value):
        if value:
            match = DAILYMOTION_BASE_URL.match(value)
            if match:
                value = match.groups()[0]
        self._video_id = value


@utility_config(name='dailymotion',
                provides=IExternalVideoProvider)
class DailymotionVideoProvider:
    """Dailymotion video provider"""

    label = _("Dailymotion")
    weight = 20

    settings_interface = IDailymotionVideoSettings


@adapter_config(required=IExternalVideo,
                provides=IDailymotionVideoSettings)
def dailymotion_video_settings(context):
    """Dailymotion video settings factory"""
    return external_video_settings(context, provider_name='dailymotion')


@adapter_config(required=DailymotionVideoProvider,
                provides=IExternalVideoSettings)
def dailymotion_video_provider_settings_factory(context):
    """Dailymotion video provider settings factory"""
    return IDailymotionVideoSettings



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery module

This module defines persistent components and adapters used to handle medias
galleries.
"""

from pyramid.events import subscriber
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent, \
    ObjectModifiedEvent
from zope.location import locate
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

from pyams_catalog.utils import index_object
from pyams_content.component.gallery.interfaces import GALLERY_CONTAINER_KEY, GALLERY_RENDERERS, \
    IBaseGallery, IGallery, IGalleryContainer, IGalleryFile, IGalleryItem, IGalleryTarget
from pyams_content.component.illustration import VirtualIllustration
from pyams_content.component.illustration.interfaces import IBaseIllustration
from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.feature.renderer import RenderedContentMixin, RenderersVocabulary
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_content.shared.common import IWfSharedContent
from pyams_file.interfaces import IBaseImageFile
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config
from pyams_utils.list import boolean_iter
from pyams_utils.registry import get_current_registry
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config


__docformat__ = 'restructuredtext'


@implementer(IGalleryContainer)
class GalleryContainer(BTreeOrderedContainer):
    """Gallery medias container"""

    def get_visible_medias(self):
        """Visible medias iterator"""
        yield from filter(lambda x: IGalleryFile(x).visible, self.values())

    def get_visible_images(self):
        """Visible images iterator"""
        yield from filter(lambda x: IBaseImageFile.providedBy(x.data),
                          self.get_visible_medias())


@factory_config(provided=IBaseGallery)
class BaseGallery(RenderedContentMixin, GalleryContainer):
    """Base gallery persistent class"""

    renderer = FieldProperty(IBaseGallery['renderer'])


@factory_config(provided=IGallery)
class Gallery(BaseGallery):
    """Gallery persistent class"""

    title = FieldProperty(IGallery['title'])
    description = FieldProperty(IGallery['description'])


@adapter_config(required=IGalleryTarget,
                provides=IGallery)
def gallery_adapter(target):
    """Gallery container adapter"""
    return get_annotation_adapter(target, GALLERY_CONTAINER_KEY, IGallery,
                                  name='++gallery++')


@adapter_config(name='gallery',
                required=IGalleryTarget,
                provides=ITraversable)
class GalleryContainerNamespace(ContextAdapter):
    """++gallery++ container namespace traverser"""

    def traverse(self, name, furtherPath=None):
        """Gallery traverser"""
        return get_current_registry().queryAdapter(self.context, IGallery, name=name)


@adapter_config(name='gallery',
                required=IGalleryTarget,
                provides=ISublocations)
class GalleryContainerSublocations(ContextAdapter):
    """Galleries container sub-locations"""

    def sublocations(self):
        """Sub-locations getter"""
        yield from IGallery(self.context).values()


@adapter_config(required=IGallery,
                provides=IViewContextPermissionChecker)
class GalleryPermissionChecker(ContextAdapter):
    """Gallery permission checker"""

    @property
    def edit_permission(self):
        """Edit permission getter"""
        content = get_parent(self.context, IWfSharedContent)
        if content is not None:
            return IViewContextPermissionChecker(content).edit_permission
        return MANAGE_SITE_ROOT_PERMISSION


@subscriber(IObjectAddedEvent, context_selector=IGallery)
@subscriber(IObjectModifiedEvent, context_selector=IGallery)
@subscriber(IObjectRemovedEvent, context_selector=IGallery)
def handle_gallery_event(event):
    """Handle event for added, modified or removed gallery"""
    gallery = event.object
    if IBaseParagraph.providedBy(gallery):
        # there is another event subscriber for paragraphs,
        # so don't trigger event twice !
        return
    content = get_parent(gallery, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@vocabulary_config(name=GALLERY_RENDERERS)
class GalleryRenderersVocabulary(RenderersVocabulary):
    """Gallery renderers vocabulary"""

    content_interface = IBaseGallery


#
# Illustrations adapters
#

@adapter_config(required=IBaseGallery,
                provides=IBaseIllustration)
def gallery_illustration_adapter(gallery):
    """Gallery illustration adapter"""
    has_medias, medias = boolean_iter(gallery.get_visible_medias())
    if has_medias:
        return IBaseIllustration(next(medias).data, None)
    return None


@adapter_config(required=IGalleryFile,
                provides=IBaseIllustration)
def gallery_file_illustration_adapter(file):
    """Gallery file illustration adapter"""
    illustration = VirtualIllustration(file.data)
    illustration.title = (file.title or {}).copy()
    illustration.alt_title = (file.alt_title or {}).copy()
    illustration.author = file.author
    return illustration



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.search module

This module defines search folder class and adapters.
"""

from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.feature.search.interfaces import ISearchFolder
from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.view import WfView
from pyams_portal.interfaces import IPortalContext, IPortalFooterContext, IPortalHeaderContext
from pyams_security.interfaces import IDefaultProtectionPolicy, IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config

from pyams_content import _
from pyams_utils.request import check_request


@factory_config(ISearchFolder)
@implementer(IDefaultProtectionPolicy,
             IIllustrationTarget, ILinkIllustrationTarget,
             IPortalContext, IPortalHeaderContext, IPortalFooterContext, IPreviewTarget)
class SearchFolder(WfView):
    """Search folder"""

    content_name = _("Search folder")

    handle_short_name = True
    handle_content_url = False
    handle_header = True
    handle_description = True

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''

    order_by = FieldProperty(ISearchFolder['order_by'])
    visible_in_list = FieldProperty(ISearchFolder['visible_in_list'])
    navigation_title = FieldProperty(ISearchFolder['navigation_title'])

    selected_content_types = FieldProperty(ISearchFolder['selected_content_types'])
    selected_datatypes = FieldProperty(ISearchFolder['selected_datatypes'])

    @staticmethod
    def is_deletable():
        return True

    def get_results(self, context, sort_index=None, reverse=None, limit=None,
                    start=0, length=None, ignore_cache=False, get_count=False, request=None,
                    aggregates=None, settings=None, **kwargs):
        if not ignore_cache:
            if request is None:
                request = check_request()
            ignore_cache = bool(request.params)
        return super().get_results(context, sort_index, reverse, limit, start,
                                   length, ignore_cache, get_count, request,
                                   aggregates, settings, **kwargs)


@adapter_config(required=ISearchFolder,
                provides=IViewContextPermissionChecker)
class SearchFolderPermissionChecker(ContextAdapter):
    """Search folder permission checker"""

    edit_permission = MANAGE_SITE_PERMISSION



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.zmi.dashboard module

This module provides components which are used in all dashboards.
"""

from zope.interface import Interface, implementer

from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.site.interfaces import ISiteLink
from pyams_content.zmi.interfaces import IDashboardColumn, IDashboardContentLabel, \
    IDashboardContentModifier, IDashboardContentNumber, IDashboardContentOwner, \
    IDashboardContentStatus, IDashboardContentStatusDatetime, IDashboardContentTimestamp, \
    IDashboardContentType, IDashboardContentVersion, IDashboardContentVisibility, IDashboardTable
from pyams_sequence.interfaces import ISequentialIntIds
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn
from pyams_utils.adapter import adapter_config
from pyams_utils.data import ObjectDataManagerMixin
from pyams_utils.interfaces import ICacheKeyValue, MISSING_INFO
from pyams_utils.registry import get_utility
from pyams_utils.request import request_property
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.table import I18nColumnMixin, NameColumn, VisibilityColumn
from pyams_zmi.utils import get_object_label


__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(IDashboardColumn)
class DashboardColumnMixin(ObjectDataManagerMixin):  # pylint: disable=no-member
    """Base dashboard column mixin"""

    interface = None

    @request_property(key=None)
    def get_target(self, obj):
        """Row target getter"""
        item_key = ICacheKeyValue(obj)
        target = self.table.rows_state.get(item_key)
        if target is None:
            target = obj
        return target

    def get_value(self, obj):
        """Column value getter"""
        target = self.get_target(obj)
        registry = self.request.registry
        value = registry.queryMultiAdapter((target, self.request, self), self.interface)
        if value is None:
            value = registry.queryAdapter(target, self.interface)
        return value or MISSING_INFO


@adapter_config(required=IDashboardColumn,
                provides=ICacheKeyValue)
def dashboard_column_cache_key(column: IDashboardColumn):
    """Dashboard column cache key"""
    return ICacheKeyValue(column.table)


class DashboardVisibilityColumn(DashboardColumnMixin, VisibilityColumn):
    """Dashboard visibility column"""

    interface = IDashboardContentVisibility
    permission = MANAGE_CONTENT_PERMISSION

    weight = 5

    active_icon_hint = _("Switch element visibility")
    inactive_icon_hint = _("Visible element?")

    def get_icon(self, item):
        """Icon getter"""
        return self.get_value(item)[1]

    def get_icon_hint(self, item):
        """Icon hint getter"""
        translate = self.request.localizer.translate
        if ISiteLink.providedBy(item) and self.has_permission(item):
            return translate(self.active_icon_hint)
        return translate(self.inactive_icon_hint)

    def render_cell(self, item):
        """Cell renderer"""
        active, icon = self.get_value(item)
        if not active:
            return self.get_icon(item)
        return super().render_cell(item)


@adapter_config(name='name',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardLabelColumn(DashboardColumnMixin, NameColumn):
    """Dashboard label column"""

    i18n_header = _("Title")
    interface = IDashboardContentLabel

    css_classes = {
        'td': 'text-truncate'
    }

    def get_value(self, obj):
        label = super().get_value(obj)
        if label == MISSING_INFO:
            target = self.get_target(obj)
            label = get_object_label(target, self.request, self.table)
        return label or MISSING_INFO


@adapter_config(name='content-type',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentTypeColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content type column"""

    i18n_header = _("Data type")
    interface = IDashboardContentType
    weight = 15


@adapter_config(name='sequence',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentNumberColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content number column"""

    @property
    def i18n_header(self):
        """Header getter"""
        sequence = get_utility(ISequentialIntIds)
        return self.request.localizer.translate(_('#{}')).format(sequence.prefix)

    interface = IDashboardContentNumber
    weight = 20


@adapter_config(name='status',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentStatusColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content status column"""

    i18n_header = _("Status")
    css_classes = {
        'td': 'text-nowrap'
    }
    interface = IDashboardContentStatus
    weight = 25


@adapter_config(name='status-datetime',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentStatusDatetimeColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content status datetime column"""

    i18n_header = _("Status datetime")
    css_classes = {
        'td': 'text-nowrap'
    }
    interface = IDashboardContentStatusDatetime
    weight = 30


@adapter_config(name='version',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentVersionColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content status version column"""

    i18n_header = _("Version")
    interface = IDashboardContentVersion
    weight = 35


@adapter_config(name='status-principal',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentStatusPrincipalColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content status modifier column"""

    i18n_header = _("Modifier")
    css_classes = {
        'td': 'text-nowrap'
    }
    interface = IDashboardContentModifier
    weight = 40


@adapter_config(name='owner',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentOwnerColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content owner column"""

    i18n_header = _("Owner")
    css_classes = {
        'td': 'text-nowrap'
    }
    interface = IDashboardContentOwner
    weight = 45


@adapter_config(name='timestamp',
                required=(Interface, IAdminLayer, IDashboardTable),
                provides=IColumn)
class DashboardContentTimestampColumn(DashboardColumnMixin, I18nColumnMixin, GetAttrColumn):
    """Dashboard content timestamp column"""

    i18n_header = _("Last update")
    css_classes = {
        'td': 'text-nowrap'
    }
    interface = IDashboardContentTimestamp
    weight = 50



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.glossary.skin module

This module defines glossary renderers and views.
"""

import logging
import pickle

from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPNotFound
from zope.interface import implementer

from pyams_content.component.thesaurus import ITagsManager
from pyams_content.feature.glossary import GLOSSARY_CACHE_KEY, GLOSSARY_CACHE_NAME, \
    GLOSSARY_CACHE_NAMESPACE, GLOSSARY_CACHE_REGION, get_glossary_automaton
from pyams_content.feature.glossary.interfaces import REQUEST_GLOSSARY_KEY
from pyams_content.feature.glossary.skin.interfaces import IThesaurusTermRenderer
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces.base import VIEW_PERMISSION
from pyams_skin.interfaces.view import IModalPage
from pyams_template.template import layout_config, template_config
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.cache import get_cache
from pyams_utils.interfaces.text import IHTMLRenderer


__docformat__ = 'restructuredtext'


LOGGER = logging.getLogger('PyAMS (content)')


@adapter_config(name='glossary',
                required=(str, IPyAMSUserLayer),
                provides=IHTMLRenderer)
class GlossaryHTMLRenderer(ContextRequestAdapter):
    """Glossary HTML renderer"""

    def render(self):
        """Glossary terms renderer"""
        source = self.context
        # check tags manager configuration
        manager = ITagsManager(self.request.root, None)
        if (manager is None) or not manager.enable_glossary:
            return source
        # get glossary automaton
        glossary_cache = get_cache(GLOSSARY_CACHE_NAME, GLOSSARY_CACHE_REGION,
                                   GLOSSARY_CACHE_NAMESPACE)
        try:
            LOGGER.debug("Loading glossary automaton...")
            automaton = glossary_cache.get_value(GLOSSARY_CACHE_KEY)
        except KeyError:
            LOGGER.debug("Automaton not found, loading new one...")
            automaton = get_glossary_automaton(self.request.root)
        else:
            automaton = pickle.loads(automaton)
        if automaton is None:
            LOGGER.debug("Missing automaton, skipping HTML conversion")
            return source
        LOGGER.debug("Automaton loaded with {} terms!".format(len(automaton)))
        #  iterate over automaton entries
        annotations = self.request.annotations
        found_terms = annotations.get(REQUEST_GLOSSARY_KEY) or set()
        last_found_index = 0
        marker = '<span class="thesaurus-term">{}</span>'
        marker_length = len(marker) - 2
        for position, text in automaton.iter(self.context):
            if (text in found_terms) or (position <= last_found_index):
                continue
            LOGGER.debug("Found term '{}' at position {}".format(text, position))
            count = len(found_terms)
            offset = marker_length * count
            start_offset = position + offset - len(text)
            if source[start_offset] in '<>':
                LOGGER.debug("Already tagged term, skipping...")
                continue
            source = source[0:start_offset + 1] + marker.format(text) + \
                source[position + offset + 1:]
            found_terms.add(text)
            last_found_index = position
        if last_found_index:
            self.request.annotations[REQUEST_GLOSSARY_KEY] = found_terms
        return source


class BaseGlossaryViewMixin:
    """Base glossary mixin view"""

    @reify
    def glossary(self):
        """Glossary getter"""
        tags_manager = ITagsManager(self.request.root, None)
        if tags_manager is None:
            raise HTTPNotFound("Can't find tags manager!")
        glossary = tags_manager.glossary
        if glossary is None:
            raise HTTPNotFound("Can't find glossary!")
        return glossary

    @property
    def title(self):
        """Title getter"""
        return self.glossary.title


@pagelet_config(name='get-glossary.html',
                layer=IPyAMSUserLayer,
                permission=VIEW_PERMISSION)
@template_config(template='templates/glossary.pt',
                 layer=IPyAMSUserLayer)
class GlossaryView(BaseGlossaryViewMixin, PortalContextIndexPage):
    """Glossary view"""

    def update(self):
        super().update()
        self.request.context = self.glossary


@template_config(template='templates/glossary-term.pt',
                 layer=IPyAMSUserLayer)
class BaseGlossaryTermViewMixin(BaseGlossaryViewMixin):
    """Base glossary term view mixin class"""

    @reify
    def term(self):
        """Term getter"""
        label = self.request.params.get('term')
        if not label:
            raise HTTPNotFound("No provided glossary term!")
        term = self.glossary.terms.get(label)
        if term is None:
            raise HTTPNotFound("Can't find provided term in glossary!")
        return term

    @property
    def renderers(self):
        registry = self.request.registry
        for name, renderer in sorted(registry.getAdapters((self.term, self.request, self),
                                                          IThesaurusTermRenderer),
                                     key=lambda x: x[1].weight):
            renderer.update()
            yield renderer

    @property
    def is_modal(self):
        return IModalPage.providedBy(self)


@pagelet_config(name='get-glossary-term-page.html',
                layer=IPyAMSUserLayer,
                permission=VIEW_PERMISSION)
class GlossaryTermPage(BaseGlossaryTermViewMixin, PortalContextIndexPage):
    """Glossary term page view"""

    def update(self):
        super().update()
        self.request.context = self.term


@pagelet_config(name='get-glossary-term.html',
                layer=IPyAMSUserLayer,
                permission=VIEW_PERMISSION)
@layout_config(template='templates/term-layout.pt',
               layer=IPyAMSUserLayer)
@implementer(IModalPage)
class GlossaryTermModalPage(BaseGlossaryTermViewMixin):
    """Glossary term modal page view"""

    modal_class = 'modal-dialog modal-dialog-scrollable modal-xl'



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.cards.interfaces module

Bootstrap cards paragraph interfaces.
"""

from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.component.paragraph.interfaces import ParagraphRendererChoice
from pyams_portal.portlets.cards import ICardsContainer

__docformat__ = 'restructuredtext'

from pyams_content import _


CARDS_PARAGRAPH_TYPE = 'cards'
CARDS_PARAGRAPH_NAME = _("Bootstrap cards")
CARDS_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.cards.renderers'
CARDS_PARAGRAPH_ICON_CLASS = 'fas fa-clipboard-list'


class ICardsParagraph(ICardsContainer, IBaseParagraph):
    """Cards paragraph interface"""

    renderer = ParagraphRendererChoice(description=_("Presentation template used for cards"),
                                       renderers=CARDS_PARAGRAPH_RENDERERS)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.contentprovider.interfaces import IContentProvider
from zope.interface import Interface

from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.tales import ITALESExtension
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config

__docformat__ = 'restructuredtext'


@contentprovider_config(name='pyams_content.card.datatype',
                        layer=IPyAMSLayer, view=Interface)
@template_config(template='templates/card-datatype.pt', layer=IPyAMSLayer)
class SharedContentDatatypeProvider(ViewContentProvider):
    """Shared content datatype content provider"""

    label = None

    def update(self):
        super().update()
        tsc = IWfTypedSharedContent(self.context, None)
        if tsc is not None:
            datatype = tsc.get_data_type()
            if (datatype is not None) and datatype.display_as_tag:
                self.label = II18n(datatype).query_attributes_in_order(('navigation_label', 'label'),
                                                                       request=self.request)


@adapter_config(name='pyams_card_datatype',
                required=(Interface, Interface, Interface),
                provides=ITALESExtension)
class DataTypeContentProvider(ContextRequestViewAdapter):
    """Datatype content provider"""

    def render(self, context=None):
        if context is None:
            context = self.context
        provider = self.request.registry.queryMultiAdapter((context, self.request, self.view),
                                                           IContentProvider,
                                                           name='pyams_content.card.datatype')
        if provider is None:
            return ''
        provider.update()
        return provider.render()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.search.zmi module

This module defines management components for search folders.
"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_content import _
from pyams_content.feature.search import ISearchFolder
from pyams_content.interfaces import CREATE_CONTENT_PERMISSION, MANAGE_CONTENT_PERMISSION, MANAGE_SITE_PERMISSION
from pyams_content.shared.site.interfaces import ISiteContainer
from pyams_content.shared.site.zmi.folder import ISiteFolderAddFormFields, SiteFolderAddForm
from pyams_content.shared.site.zmi.interfaces import ISiteTreeTable
from pyams_content.shared.site.zmi.widget.folder import SiteManagerFoldersSelectorFieldWidget
from pyams_content.shared.view.zmi import ViewPropertiesEditForm, ViewPropertiesGroup
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security_views.zmi.interfaces import IObjectSecurityMenu
from pyams_skin.interfaces.viewlet import IBreadcrumbItem, IHelpViewletManager
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_skin.viewlet.menu import MenuDivider, MenuItem
from pyams_utils.adapter import NullAdapter, adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IContextActionsDropdownMenu, \
    IContextAddingsViewletManager, IMenuHeader
from pyams_zmi.table import TableElementEditor


@viewlet_config(name='add-search-folder.divider',
                context=ISiteContainer, layer=IAdminLayer, view=ISiteTreeTable,
                manager=IContextAddingsViewletManager, weight=89,
                permission=MANAGE_SITE_PERMISSION)
class SearchFolderAddMenuDivider(MenuDivider):
    """Search folder add menu divider"""


@viewlet_config(name='add-search-folder.menu',
                context=ISiteContainer, layer=IAdminLayer, view=ISiteTreeTable,
                manager=IContextAddingsViewletManager, weight=90,
                permission=MANAGE_SITE_PERMISSION)
class SearchFolderAddMenu(MenuItem):
    """Search folder add menu"""

    label = _("Add search folder...")
    icon_class = 'fas fa-search'

    href = 'add-search-folder.html'
    modal_target = True


class ISearchFolderAddFormFields(ISiteFolderAddFormFields):
    """Search folder add form fields interface"""


@ajax_form_config(name='add-search-folder.html',
                  context=ISiteContainer, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_PERMISSION)
class SearchFolderAddForm(SiteFolderAddForm):
    """Search folder add form"""

    subtitle = _("New search folder")
    legend = _("New search folder properties")

    fields = Fields(ISearchFolderAddFormFields)
    fields['parent'].widget_factory = SiteManagerFoldersSelectorFieldWidget
    content_factory = ISearchFolder

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'parent' in self.widgets:
            self.widgets['parent'].permission = MANAGE_SITE_PERMISSION


@adapter_config(required=(ISearchFolder, IAdminLayer, Interface),
                provides=IObjectLabel)
def search_folder_label(context, request, view):
    """Search folder label"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(ISearchFolder, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def search_folder_management_menu_header(context, request, view, manager):
    """Search folder management menu header"""
    return request.localizer.translate(_("Search folder management"))


@adapter_config(required=(ISearchFolder, IAdminLayer, Interface),
                provides=ITableElementEditor)
class SearchFolderTableElementEditor(TableElementEditor):
    """Search folder table element editor"""

    view_name = 'admin'
    modal_target = False


@adapter_config(required=(ISearchFolder, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class SearchFolderBreadcrumbs(BreadcrumbItem):
    """Search folder breadcrumb item"""

    @property
    def label(self):
        return II18n(self.context).query_attribute('short_name', request=self.request)

    view_name = 'admin'


@adapter_config(name='view-properties',
                required=(ISearchFolder, IAdminLayer, ViewPropertiesEditForm),
                provides=IGroup)
class SearchFolderPropertiesGroup(ViewPropertiesGroup):
    """View properties group"""

    legend = _("Folder search settings")

    fields = Fields(ISearchFolder).select('selected_content_types', 'selected_datatypes',
                                          'excluded_content_types', 'excluded_datatypes', 'allow_user_params',
                                          'order_by', 'reversed_order', 'limit', 'age_limit')


@viewlet_config(name='view-properties.help',
                context=ISearchFolder, layer=IAdminLayer, view=ViewPropertiesGroup,
                manager=IHelpViewletManager, weight=10)
class SearchFolderPropertiesHelp(NullAdapter):
    """Search folder properties help"""


@adapter_config(name='navigation',
                required=(ISearchFolder, IAdminLayer, IPropertiesEditForm),
                provides=IGroup)
class SearchFolderPropertiesEditFormNavigationGroup(FormGroupSwitcher):
    """Search folder properties edit form navigation group"""

    legend = _("Navigation properties")
    weight = 10

    fields = Fields(ISearchFolder).select('visible_in_list', 'navigation_title')


@viewlet_config(name='change-owner.menu',
                context=ISearchFolder, layer=IAdminLayer, view=Interface,
                manager=IObjectSecurityMenu, weight=300,
                permission=MANAGE_SITE_PERMISSION)
class SearchFolderOwnerChangeMenu(NullAdapter):
    """Search folder owner change menu is disabled"""


@viewlet_config(name='duplicate-content.divider',
                context=ISearchFolder, layer=IAdminLayer,
                manager=IContextActionsDropdownMenu, weight=49,
                permission=CREATE_CONTENT_PERMISSION)
class SearchFolderDuplicateMenuDivider(NullAdapter):
    """Search folder duplication menu divider is disabled"""


@viewlet_config(name='duplicate-content.menu',
                context=ISearchFolder, layer=IAdminLayer,
                manager=IContextActionsDropdownMenu, weight=50,
                permission=CREATE_CONTENT_PERMISSION)
class SearchFolderDuplicateMenu(NullAdapter):
    """Search folder duplication menu item is disabled"""


@viewlet_config(name='content-review.menu',
                context=ISearchFolder, layer=IAdminLayer,
                manager=IContextActionsDropdownMenu, weight=10,
                permission=MANAGE_CONTENT_PERMISSION)
class SearchFolderReviewMenu(NullAdapter):
    """Search folder review menu is disabled"""



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.zmi module


"""
from cgi import FieldStorage

from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPForbidden, HTTPServerError
from pyramid.view import view_config
from zope.interface import implementer
from zope.location import locate
from zope.schema._bootstrapinterfaces import WrongType

from pyams_content.component.gallery import IGallery, IGalleryFile, IGalleryTarget
from pyams_content.component.gallery.interfaces import IGalleryContainer
from pyams_content.component.gallery.zmi.helpers import get_json_gallery_refresh_event
from pyams_content.component.gallery.zmi.interfaces import IGalleryMediasView
from pyams_content.component.paragraph.zmi import get_json_paragraph_toolbar_refresh_event
from pyams_file.file import get_magic_content_type
from pyams_file.interfaces.archive import IArchiveExtractor
from pyams_form.interfaces.form import IInnerSubForm
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_layer.skin import apply_skin
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.permission import get_edit_permission
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import create_object, factory_config, get_object_factory
from pyams_utils.interfaces import ICacheKeyValue, MISSING_INFO
from pyams_utils.registry import query_utility
from pyams_utils.rest import http_error
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.skin import AdminSkin
from pyams_zmi.view import InnerAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseGalleryMediasViewlet(Viewlet):
    """Base gallery medias viewlet"""

    @reify
    def gallery(self):
        """Gallery getter"""
        return IGalleryContainer(self.context)

    @property
    def gallery_name(self):
        """Gallery name getter"""
        return ICacheKeyValue(self.gallery)

    @property
    def edit_permission(self):
        """Edit permission getter"""
        return get_edit_permission(self.request, self.gallery, self)

    @property
    def medias(self):
        """Gallery medias getter"""
        return self.gallery.values()

    def get_title(self, media):
        """Media title getter"""
        return II18n(media).query_attribute('title', request=self.request)


@adapter_config(name='gallery-medias',
                required=(IGalleryContainer, IAdminLayer, IPropertiesEditForm),
                provides=IInnerSubForm, force_implements=False)
@template_config(template='templates/gallery-medias.pt', layer=IAdminLayer)
@factory_config(IGalleryMediasView)
class GalleryMediasViewlet(BaseGalleryMediasViewlet):
    """Gallery medias viewlet"""

    def __init__(self, context, request, view, manager=None):
        super().__init__(context, request, view, manager)


@view_config(name='upload-medias-files.json',
             context=IGalleryContainer, request_type=IPyAMSLayer,
             request_method='POST', renderer='json', xhr=True, require_csrf=False)
def upload_medias_files(request):
    """Upload medias files"""
    container = IGalleryContainer(request.context)
    permission_checker = IViewContextPermissionChecker(container, None)
    if permission_checker is None:
        return http_error(request, HTTPServerError, "Missing context permission")
    if not request.has_permission(permission_checker.edit_permission, context=container):
        return http_error(request, HTTPForbidden)
    for name in request.params.keys():
        input_file = request.params.get(name)
        if isinstance(input_file, FieldStorage):
            value = input_file.value
            content_type = get_magic_content_type(value)
            extractor = query_utility(IArchiveExtractor, name=content_type)
            if extractor is not None:
                contents = extractor.get_contents(value)
            else:
                contents = ((value, input_file.filename),)
            for content, filename in contents:
                try:
                    gallery_file = create_object(IGalleryFile)
                    locate(gallery_file, container)
                    gallery_file.data = filename, content
                    gallery_file.author = MISSING_INFO
                except WrongType:
                    continue
                else:
                    container.append(gallery_file, notify=False)
    apply_skin(request, AdminSkin)
    result_view = create_object(IGalleryMediasView, context=container, request=request, view=None)
    result_view.update()
    result = {
        'status': 'success',
        'callbacks': [
            get_json_gallery_refresh_event(container, request, result_view)
        ]
    }
    event = get_json_paragraph_toolbar_refresh_event(container, request)
    if event is not None:
        result.setdefault('callbacks', []).append(event)
    return result


@view_config(name='set-medias-order.json',
             context=IGalleryContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def set_medias_order(request):
    """Medias ordering view"""

    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success',
        'closeForm': False
    }


@view_config(name='switch-media-visibility.json',
             context=IGalleryContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_media_visibility(request):
    """Media visibility switch view"""
    return switch_element_attribute(request)


@view_config(name='remove-media.json',
             context=IGalleryContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def remove_media(request):
    """Media remover view"""
    result = delete_container_element(request, container_factory=IGalleryContainer)
    if result.get('status') == 'success':
        apply_skin(request, AdminSkin)
        gallery = request.context
        result = {
            'status': 'success',
            'callbacks': [
                get_json_gallery_refresh_event(gallery, request, None)
            ]
        }
        event = get_json_paragraph_toolbar_refresh_event(gallery, request)
        if event is not None:
            result.setdefault('callbacks', []).append(event)
        result.setdefault('handle_json', True)
    return result


@viewlet_config(name='gallery.menu',
                context=IGalleryTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=20,
                permission=VIEW_SYSTEM_PERMISSION)
class GalleryMenu(NavigationMenuItem):
    """Gallery menu"""

    label = _("Medias gallery")
    href = '#medias-gallery.html'


@pagelet_config(name='medias-gallery.html',
                context=IGalleryTarget, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/gallery-view.pt', layer=IAdminLayer)
@implementer(IPropertiesEditForm)
class GalleryView(InnerAdminView):
    """Gallery view"""

    title = _("Medias gallery")

    medias_view = None

    def update(self):
        super().update()
        medias_view_factory = get_object_factory(IGalleryMediasView)
        if medias_view_factory is not None:
            gallery = IGallery(self.context)
            self.medias_view = medias_view_factory(gallery, self.request, self)
            self.medias_view.update()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.portlet.skin module

This module defines illustration portlet renderers.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import IIllustration
from pyams_content.component.illustration.portlet.interfaces import IIllustrationPortletContent, \
    IIllustrationPortletSettings
from pyams_content.component.illustration.portlet.skin.interfaces import \
    IIllustrationPortletBaseRendererSettings, IIllustrationPortletDefaultRendererSettings, \
    IIllustrationPortletSideRendererSettings
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import HIDDEN_RENDERER_NAME, IPortalContext, IPortletCSSClass, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IIllustrationPortletBaseRendererSettings)
class IllustrationPortletBaseRendererSettings(Persistent, Contained):
    """Illustration portlet base renderer settings"""

    thumb_selection = FieldProperty(
        IIllustrationPortletBaseRendererSettings['thumb_selection'])


class BaseIllustrationPortletRenderer(PortletRenderer):
    """Base illustration portlet renderer"""

    @property
    def illustration(self):
        """Illustration getter"""
        registry = self.request.registry
        context = self.request.context
        for illustration in (
                registry.queryMultiAdapter((context, self.request, self.view), IIllustrationPortletContent),
                IIllustration(context, None),
                registry.queryMultiAdapter((self.context, self.request, self.view), IIllustrationPortletContent),
                IIllustration(self.context, None)):
            if (illustration is not None) and \
                    (illustration.renderer != HIDDEN_RENDERER_NAME) and \
                    illustration.has_data():
                return illustration
        return None


#
# Illustration portlet default renderer
#

@factory_config(IIllustrationPortletDefaultRendererSettings)
class IllustrationPortletDefaultRendererSettings(IllustrationPortletBaseRendererSettings):
    """Illustration portlet default renderer settings"""

    display_title = FieldProperty(IIllustrationPortletDefaultRendererSettings['display_title'])
    display_author = FieldProperty(IIllustrationPortletDefaultRendererSettings['display_author'])
    display_description = FieldProperty(
        IIllustrationPortletDefaultRendererSettings['display_description'])


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface, IIllustrationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class IllustrationPortletDefaultRenderer(BaseIllustrationPortletRenderer):
    """Illustration portlet default renderer"""

    label = _("Full width illustration (default)")

    settings_interface = IIllustrationPortletDefaultRendererSettings
    weight = 10


#
# Illustration portlet side renderers
#

@factory_config(IIllustrationPortletSideRendererSettings)
class IllustrationPortletSideRendererSettings(IllustrationPortletBaseRendererSettings):
    """Illustration portlet base renderer settings"""

    thumb_selection = FieldProperty(
        IIllustrationPortletSideRendererSettings['thumb_selection'])
    zoom_on_click = FieldProperty(
        IIllustrationPortletSideRendererSettings['zoom_on_click'])

    def get_css_cols(self, side):
        """Get CSS cols matching current selections"""
        return ' '.join([
            f"float-{'{}-'.format(device) if device != 'xs' else ''}"
            f"{'none' if selection.cols==12 else side} "
            f"col-{'{}-'.format(device) if device != 'xs' else ''}{selection.cols} "
            f"mx-{'{}-'.format(device) if device != 'xs' else ''}"
            f"{'0' if selection.cols==12 else '3'}"
            for device, selection in self.thumb_selection.items()
        ])


@template_config(template='templates/illustration-side.pt', layer=IPyAMSLayer)
class IllustrationPortletSideRenderer(BaseIllustrationPortletRenderer):
    """Illustration portlet side renderer"""

    settings_interface = IIllustrationPortletSideRendererSettings


@adapter_config(name='float-left',
                required=(IPortalContext, IPyAMSLayer, Interface, IIllustrationPortletSettings),
                provides=IPortletRenderer)
class IllustrationPortletLeftFloatRenderer(IllustrationPortletSideRenderer):
    """Left floating illustration portlet renderer"""

    label = _("Floating illustration to the left")
    css_class = 'float-right'

    weight = 20


@adapter_config(name='float-left',
                required=(IPortalContext, IPyAMSLayer, Interface,
                          IIllustrationPortletSideRendererSettings),
                provides=IPortletCSSClass)
def illustration_portlet_left_float_css_class(context, request, view, renderer_settings):
    """Left floating illustration portlet renderer CSS class"""
    return f"{renderer_settings.get_css_cols('left')} mb-3 px-0"


@adapter_config(name='float-right',
                required=(IPortalContext, IPyAMSLayer, Interface, IIllustrationPortletSettings),
                provides=IPortletRenderer)
class IllustrationPortletRightFloatRenderer(IllustrationPortletSideRenderer):
    """Right floating illustration portlet renderer"""

    label = _("Floating illustration to the right")
    css_class = 'float-right'

    weight = 30


@adapter_config(name='float-right',
                required=(IPortalContext, IPyAMSLayer, Interface,
                          IIllustrationPortletSideRendererSettings),
                provides=IPortletCSSClass)
def illustration_portlet_right_float_css_class(context, request, view, renderer_settings):
    """Right floating illustration portlet renderer CSS class"""
    return f"{renderer_settings.get_css_cols('right')} mb-3 px-0"



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.paragraph.portlet.skin.interfaces module

This module defines a single interface which can be used to define custom
paragraph container portlet renderers.
"""

from zope.interface import Attribute

from pyams_portal.interfaces import IPortletContentProvider


__docformat__ = 'restructuredtext'


class IParagraphContainerPortletRenderer(IPortletContentProvider):
    """Paragraph container portlet renderer interface

    This interface is used to create custom paragraphs container portlet
    renderers, which may be registered with a view name.
    """

    use_portlets_cache = Attribute("Use portlets cache?")
